/************************************************************************************************/
/*                                                                                              */
/*      <cyberHotel: Gestion Hotelera GPL>                                                      */
/*      Copyright (C) <2008> <davide05@gmail.com>                                               */
/*                                                                                              */
/*      This program is free software: you can redistribute it and/or modify                    */
/*      it under the terms of the GNU General Public License as published by                    */
/*      the Free Software Foundation, either version 3 of the License, or                       */
/*      (at your option) any later version.                                                     */
/*                                                                                              */
/*      This program is distributed in the hope that it will be useful,                         */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of                          */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                                    */ 
/*      See the GNU General Public License for more details.                                    */
/*                                                                                              */
/*      You should have received a copy of the GNU General Public License                       */
/*      along with this program.  If not, see <http://www.gnu.org/licenses/>.                   */
/*                                                                                              */
/************************************************************************************************/

package gz.cyberbrain.cyberhotel.controller;

import gz.cyberbrain.cyberhotel.model.board.to.BoardTo;
import gz.cyberbrain.cyberhotel.model.client.to.AgencyTo;
import gz.cyberbrain.cyberhotel.model.client.to.ClientTo;
import gz.cyberbrain.cyberhotel.model.client.to.CompanyTo;
import gz.cyberbrain.cyberhotel.model.client.to.ParticularTo;
import gz.cyberbrain.cyberhotel.model.configurationfacade.delegate.ConfigurationFacadeDelegate;
import gz.cyberbrain.cyberhotel.model.pricetype.to.PriceTypeTo;
import gz.cyberbrain.cyberhotel.model.request.to.RequestTo;
import gz.cyberbrain.cyberhotel.model.reservation.to.ReservationTo;
import gz.cyberbrain.cyberhotel.model.reservationfacade.delegate.ReservationFacadeDelegate;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.CancelledReservationException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.DifferentDayException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.FinishedReservationException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.PendingReservationException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.PriceNotFoundException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.RoomNotBusyException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableConceptException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableInvoiceException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableParticularException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableRoomException;
import gz.cyberbrain.cyberhotel.model.room.to.RoomTo;
import gz.cyberbrain.cyberhotel.model.spring.facade.SpringFacadeFactory;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InstanceNotFoundException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InternalErrorException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InvalidIntervalException;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.TextItem;

import java.util.Calendar;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

public class CheckOutController {

	private static ConfigurationFacadeDelegate config = SpringFacadeFactory.getConfigurationFacadeDelegate();
	private static ReservationFacadeDelegate delegate = SpringFacadeFactory.getReservationFacadeDelegate();
	private static HashMap<String, Object> attributes = new HashMap<String, Object>();

	private CheckOutController() {}

	public static Vector checkOutList() throws InternalErrorException {				
		Vector<TextItem> items = new Vector<TextItem>();

		Collection checkOuts = delegate.findAllCheckOuts();

		Iterator iter = checkOuts.iterator();

		ReservationTo checkOut;

		for (int i=0; iter.hasNext(); i++) {

			checkOut = (ReservationTo) iter.next();
			items.add(new TextItem(checkOut.getReservationId(), checkOut.getReservatorName()));

		}

		return items;
	}

	public static void doCheckOut(Long reservationId, String reservatorName, String reservatorCif, String reservatorAccount, 
			String reservatorCard, String reservatorPhone, String reservationStatus, Calendar incomingDate, 
			Calendar expectedOutgoingDate, Calendar trueOutgoingDate, Long payer, Long agency, Long company, 
			String notes, boolean byHours, Calendar finalTrueOutgoingDate) throws DifferentDayException, PendingReservationException, 
			CancelledReservationException, FinishedReservationException, InvalidIntervalException, UnaceptableRoomException, 
			UnaceptableConceptException, UnaceptableParticularException, UnaceptableInvoiceException, RoomNotBusyException,  
			PriceNotFoundException, InstanceNotFoundException, InternalErrorException {

		delegate.checkOut(new ReservationTo(reservationId, reservatorName, reservatorCif, reservatorAccount, 
				reservatorCard, reservatorPhone, reservationStatus, incomingDate, expectedOutgoingDate, 
				trueOutgoingDate, payer, agency, company, notes, byHours), finalTrueOutgoingDate);

	}

	public static Collection reservationRequests(Long reservationId) throws InternalErrorException {
		return delegate.reservationRequests(reservationId);
	}

	public static HashMap findCheckOut(Long reservationId) throws InstanceNotFoundException, InternalErrorException {

		ReservationTo checkOut = delegate.findAReservation(reservationId);

		attributes.put("id", reservationId);
		attributes.put("reservatorName", checkOut.getReservatorName());
		attributes.put("reservatorCif", checkOut.getReservatorCif());
		attributes.put("reservatorPhone", checkOut.getReservatorPhone());		
		attributes.put("reservatorAccount", checkOut.getReservatorAccount());
		attributes.put("reservatorCard", checkOut.getReservatorCard());
		attributes.put("reservationStatus", checkOut.getReservationStatus());
		attributes.put("incomingDate", checkOut.getIncomingDate());
		attributes.put("expectedOutgoingDate", checkOut.getExpectedOutgoingDate());
		attributes.put("trueOutgoingDate", checkOut.getTrueOutgoingDate());

		ClientTo payer = delegate.findAPayer(checkOut.getPayer());
		attributes.put("payer", new TextItem(payer.getClientId(), payer.getReferenceName()));

		if (checkOut.getAgency() != null) {
			AgencyTo agency = delegate.findAnAgency(checkOut.getAgency());
			attributes.put("agency", new TextItem(agency.getClientId(), agency.getName()));
		} else attributes.put("agency", new TextItem(new Long(0), ""));


		if (checkOut.getCompany() != null) {
			CompanyTo company = delegate.findACompany(checkOut.getCompany());
			attributes.put("company", new TextItem(company.getClientId(), company.getName()));
		} else attributes.put("company", new TextItem(new Long(0), ""));

		//Only for individual reservations:

		Vector request = findRequests(reservationId);

		if (request.size() == 1) {

			attributes.put("particular", ((HashMap) request.elementAt(0)).get("particular"));
			attributes.put("room", ((HashMap) request.elementAt(0)).get("room"));
			attributes.put("board", ((HashMap) request.elementAt(0)).get("board"));
			attributes.put("priceType", ((HashMap) request.elementAt(0)).get("priceType"));

		}

		attributes.put("notes", checkOut.getNotes());

		return attributes;

	}

	public static Vector findRequests(Long reservationId) throws InstanceNotFoundException, InternalErrorException {

		Collection requests = delegate.reservationRequests(reservationId);
		if (requests.size() == 0) throw new InternalErrorException(new Exception("Incorrect number of requests."));

		Vector <HashMap> requestsVector = new Vector <HashMap> ();

		Iterator iter = requests.iterator();

		for (int i=0; iter.hasNext(); i++) {

			RequestTo request = (RequestTo) iter.next();
			HashMap <String, Object> currentRequest = new HashMap <String, Object> ();

			ParticularTo particular = delegate.findAParticular(request.getParticular());
			currentRequest.put("particular", new TextItem(particular.getClientId(), particular.getReferenceName()));

			RoomTo room = config.findRoom(request.getRoom());		
			currentRequest.put("room", new TextItem(room.getRoomCode(), room.getNumber()));

			BoardTo board = config.findBoard(request.getBoard());
			currentRequest.put("board", new TextItem(board.getBoardCode(), board.getDescription()));

			PriceTypeTo priceType = config.findPriceType(request.getPriceType());
			currentRequest.put("priceType", new TextItem(priceType.getPriceTypeCode(), priceType.getDescription()));

			requestsVector.add(currentRequest);

		}

		return requestsVector;

	}

}
