/************************************************************************************************/
/*                                                                                              */
/*      <cyberHotel: Gestion Hotelera GPL>                                                      */
/*      Copyright (C) <2008> <davide05@gmail.com>                                               */
/*                                                                                              */
/*      This program is free software: you can redistribute it and/or modify                    */
/*      it under the terms of the GNU General Public License as published by                    */
/*      the Free Software Foundation, either version 3 of the License, or                       */
/*      (at your option) any later version.                                                     */
/*                                                                                              */
/*      This program is distributed in the hope that it will be useful,                         */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of                          */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                                    */ 
/*      See the GNU General Public License for more details.                                    */
/*                                                                                              */
/*      You should have received a copy of the GNU General Public License                       */
/*      along with this program.  If not, see <http://www.gnu.org/licenses/>.                   */
/*                                                                                              */
/************************************************************************************************/

package gz.cyberbrain.cyberhotel.controller;

import gz.cyberbrain.cyberhotel.model.bank.to.BankTo;
import gz.cyberbrain.cyberhotel.model.client.to.CompanyTo;
import gz.cyberbrain.cyberhotel.model.configurationfacade.delegate.ConfigurationFacadeDelegate;
import gz.cyberbrain.cyberhotel.model.reservationfacade.delegate.ReservationFacadeDelegate;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.ClientPayerException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.ClientWithInvoicesException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.InvalidPayModeException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.CompanyWithReservationsException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableAddressException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableBankException;
import gz.cyberbrain.cyberhotel.model.spring.facade.SpringFacadeFactory;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InstanceNotFoundException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InternalErrorException;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.ImageItem;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.TextItem;

import java.io.Serializable;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

public class CompanyController {

	private static ConfigurationFacadeDelegate config = SpringFacadeFactory.getConfigurationFacadeDelegate();
	private static ReservationFacadeDelegate delegate = SpringFacadeFactory.getReservationFacadeDelegate();
	private static HashMap<String, Object> attributes = new HashMap<String, Object>();
	
	private CompanyController() {}
	
	public static Vector companyList() throws InternalErrorException {				
		Vector<TextItem> items = new Vector<TextItem>();
		Collection companies = delegate.showCompanies();
		Iterator iter = companies.iterator();

		CompanyTo company;
		for (int i=0; iter.hasNext(); i++) {

			company = (CompanyTo) iter.next();
			items.add(new TextItem(company.getClientId(), company.getName()));

		}

		return items;
	}

	public static Serializable createCompany(Long address, String street, String dwellingPlace, String phone1, String phone2, 
			String fax, String email, String web, Long bank, Long card, String notes, String cif, String name, 
			String representative, String accountNumber) throws InvalidPayModeException, UnaceptableAddressException, 
									  							UnaceptableBankException, InternalErrorException {
		
		return delegate.insertACompany(new CompanyTo(null, address, street, dwellingPlace, phone1, phone2, fax, 
				email, web, bank, card, notes, cif, name, representative, accountNumber)).getClientId();
		
	}

	public static void updateCompany(Long clientId, Long address, String street, String dwellingPlace, String phone1, 
			String phone2, String fax, String email, String web, Long bank, Long card, String notes, String cif, 
			String name, String representative, String accountNumber) 
	throws  InvalidPayModeException, UnaceptableAddressException, 
			UnaceptableBankException, InstanceNotFoundException, InternalErrorException {				
		
		delegate.updateACompany(new CompanyTo(clientId, address, street, dwellingPlace, phone1, phone2, fax, 
				email, web, bank, card, notes, cif, name, representative, accountNumber));
		
	}
	
	public static void deleteCompany(Long clientId) 
		throws ClientPayerException, ClientWithInvoicesException, CompanyWithReservationsException, 
		InstanceNotFoundException, InternalErrorException {
		
		CompanyTo company = delegate.findACompany(clientId);
		delegate.removeACompany(company);
		
	}
	
	public static HashMap findCompany(Long clientId) throws InstanceNotFoundException, InternalErrorException {
		
		CompanyTo company = delegate.findACompany(clientId);
		
		attributes.put("id", clientId);		
		attributes.put("address", company.getAddress());
		attributes.put("street", company.getStreet());
		attributes.put("dwellingPlace", company.getDwellingPlace());
		attributes.put("phone1", company.getPhone1());
		attributes.put("phone2", company.getPhone2());
		attributes.put("fax", company.getFax());
		attributes.put("email", company.getEmail());
		attributes.put("web", company.getWeb());
		
		BankTo bank = config.findBank(company.getBank());
		attributes.put("bank", new ImageItem(bank.getBankId(), bank.getName(), bank.getLogo()));		
		
		attributes.put("notes", company.getNotes());
		attributes.put("cif", company.getCif());
		attributes.put("name", company.getName());		
		attributes.put("representative", company.getRepresentative());
		attributes.put("accountNumber", company.getAccountNumber());		
				
		return attributes;
				
	}

	
}
