/************************************************************************************************/
/*                                                                                              */
/*      <cyberHotel: Gestion Hotelera GPL>                                                      */
/*      Copyright (C) <2008> <davide05@gmail.com>                                               */
/*                                                                                              */
/*      This program is free software: you can redistribute it and/or modify                    */
/*      it under the terms of the GNU General Public License as published by                    */
/*      the Free Software Foundation, either version 3 of the License, or                       */
/*      (at your option) any later version.                                                     */
/*                                                                                              */
/*      This program is distributed in the hope that it will be useful,                         */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of                          */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                                    */ 
/*      See the GNU General Public License for more details.                                    */
/*                                                                                              */
/*      You should have received a copy of the GNU General Public License                       */
/*      along with this program.  If not, see <http://www.gnu.org/licenses/>.                   */
/*                                                                                              */
/************************************************************************************************/

package gz.cyberbrain.cyberhotel.controller;

import gz.cyberbrain.cyberhotel.model.client.to.ClientTo;
import gz.cyberbrain.cyberhotel.model.client.to.ParticularTo;
import gz.cyberbrain.cyberhotel.model.concept.to.ConceptTo;
import gz.cyberbrain.cyberhotel.model.configurationfacade.delegate.ConfigurationFacadeDelegate;
import gz.cyberbrain.cyberhotel.model.invoice.to.InvoiceTo;
import gz.cyberbrain.cyberhotel.model.invoicingfacade.delegate.InvoicingFacadeDelegate;
import gz.cyberbrain.cyberhotel.model.invoicingfacade.exceptions.UnaceptableOweException;
import gz.cyberbrain.cyberhotel.model.invoicingfacade.exceptions.UnaceptablePayerException;
import gz.cyberbrain.cyberhotel.model.invoicingfacade.spring.actions.PayedInvoiceException;
import gz.cyberbrain.cyberhotel.model.owe.to.OweTo;
import gz.cyberbrain.cyberhotel.model.reservationfacade.delegate.ReservationFacadeDelegate;
import gz.cyberbrain.cyberhotel.model.room.to.RoomTo;
import gz.cyberbrain.cyberhotel.model.spring.facade.SpringFacadeFactory;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InstanceNotFoundException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InternalErrorException;
import gz.cyberbrain.cyberhotel.model.util.money.MoneyOperations;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.TextItem;
import gz.cyberbrain.cyberhotel.view.swing.util.locale.Locale;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

public class InvoiceController {

	private static ConfigurationFacadeDelegate config = SpringFacadeFactory.getConfigurationFacadeDelegate();
	private static ReservationFacadeDelegate reservation = SpringFacadeFactory.getReservationFacadeDelegate();
	private static InvoicingFacadeDelegate delegate = SpringFacadeFactory.getInvoicingFacadeDelegate();
	private static HashMap <String, Object> attributes = new HashMap <String, Object> ();
	
	private InvoiceController(){}
	
	public static Vector invoiceList() {				
		Vector<TextItem> items = new Vector<TextItem>();
		Collection invoices;
		try {
			invoices = delegate.showInvoices();

			Iterator iter = invoices.iterator();

			InvoiceTo invoice;

			for (int i=0; iter.hasNext(); i++) {

				invoice = (InvoiceTo) iter.next();
				items.add(new TextItem(invoice.getId(), invoice.getInvoiceNumber() + "/" + invoice.getInvoiceSerial()));

			}

		} catch (InternalErrorException e) {
			e.printStackTrace();
		}

		return items;
	}
	
	public static Serializable createInvoice(Long invoiceNumber, String invoiceSerial, Calendar invoiceDate, 
			String shipperName, String shipperCif, String shipperStreet, Long payer, 
			String payMode, BigDecimal subtotal, BigDecimal quotaVat, BigDecimal total, String status, 
			String observations, Vector oweNumbers)
	throws UnaceptablePayerException, UnaceptableOweException, InstanceNotFoundException, InternalErrorException {
		
		return delegate.insertAnInvoice(new InvoiceTo(invoiceNumber, invoiceSerial, invoiceDate, shipperName, 
				shipperCif, shipperStreet, payer, payMode, subtotal, 
				quotaVat, total, status, observations), oweNumbers);

	}

	public static void updateInvoice(Long invoiceNumber, String invoiceSerial, Calendar invoiceDate, 
			String shipperName, String shipperCif, String shipperStreet, Long payer, 
			String payMode, BigDecimal subtotal, BigDecimal quotaVat, BigDecimal total, String status, 
			String observations, Vector oweNumbers)
	throws UnaceptablePayerException, UnaceptableOweException, InstanceNotFoundException, InternalErrorException {
		
		delegate.updateAnInvoice(new InvoiceTo(invoiceNumber, invoiceSerial, invoiceDate, shipperName, 
				shipperCif, shipperStreet, payer, payMode, subtotal, 
				quotaVat, total, status, observations), oweNumbers);

	}

	public static void deleteInvoice(Long invoiceNumber, String invoiceSerial) 
		throws PayedInvoiceException, InstanceNotFoundException, InternalErrorException {

		InvoiceTo invoice = delegate.findAnInvoice(invoiceNumber, invoiceSerial);
		delegate.deleteAnInvoice(invoice);

	}
	
	public static HashMap findInvoice(String id) throws InstanceNotFoundException, InternalErrorException {
		
		//Parse the id (invoiceNumber/invoiceSerial)
		
		int separator = id.lastIndexOf("/");
		
		String invoiceNumber = id.substring(0, separator);
		String invoiceSerial = id.substring(separator + 1, id.length());
		
		//Find and return the invoice
		
		InvoiceTo invoice = delegate.findAnInvoice(new Long(invoiceNumber), invoiceSerial);
		
		attributes.put("id", id);
		
		attributes.put("invoiceNumber", invoice.getInvoiceNumber());
		attributes.put("invoiceSerial", invoice.getInvoiceSerial());
		attributes.put("invoiceDate", invoice.getInvoiceDate());
		attributes.put("shipperName", invoice.getShipperName());
		attributes.put("shipperCif", invoice.getShipperCif());
		attributes.put("shipperStreet", invoice.getShipperStreet());
		
		ClientTo payer = delegate.findAPayer(invoice.getPayer());
		attributes.put("payer", new TextItem(payer.getClientId(), payer.getReferenceName()));
		
		attributes.put("payMode", invoice.getPayMode());
		attributes.put("subtotal", MoneyOperations.getDecimalReduction(invoice.getSubtotal()));
		attributes.put("quotaVat", MoneyOperations.getDecimalReduction(invoice.getQuotaVat()));
		attributes.put("total", MoneyOperations.getDecimalReduction(invoice.getTotal()));
		attributes.put("status", invoice.getStatus());
		attributes.put("observations", invoice.getObservations());
		
		attributes.put("invoiceOwes", loadData(new Long(invoiceNumber), invoiceSerial));

		return attributes;

	}
	
	private static Object [][] loadData(Long invoiceNumber, String invoiceSerial) throws InstanceNotFoundException, InternalErrorException {

		Object [][] data = null;

		Collection rows = null;

		rows = delegate.findInvoiceOwes(invoiceNumber, invoiceSerial);

		data = new Object[rows.size()][9];				

		Iterator iter = rows.iterator();
		
		RoomTo room;
		ParticularTo particular;
		ConceptTo concept;
		
		for (int i = 0; iter.hasNext(); i++) {
			OweTo owe = (OweTo) iter.next();
			
			room = config.findRoom(owe.getRoom());
			particular = reservation.findAParticular(owe.getParticular());
			concept = config.findConcept(owe.getConcept());
			
			data[i][0] = (Date) ((Calendar) owe.getOweDate()).getTime();
			data[i][1] = new TextItem(room.getRoomCode(), room.getNumber());
			data[i][2] = new TextItem(particular.getClientId(), particular.getReferenceName());
			data[i][3] = new TextItem(concept.getConceptCode(), concept.getDenomination());
			data[i][4] = owe.getPrice();
			data[i][5] = owe.getVat();
			data[i][6] = owe.getAmount();
			if (((String) owe.getStatus()).equals("Pendente")) data[i][7] = new TextItem("Pendente", Locale.get("OwesDialog.statusComboBox.outstanding"));
			if (((String) owe.getStatus()).equals("Facturado")) data[i][7] = new TextItem("Facturado", Locale.get("OwesDialog.statusComboBox.billed"));
			data[i][8] = (Long) ((Long) owe.getOweNumber()).longValue();
			
		}

		return data;

	}

	public static Vector outstandingInvoices() throws InternalErrorException {
		Vector<TextItem> items = new Vector<TextItem>();

		Collection invoices = delegate.findOutstandingInvoices();

		Iterator iter = invoices.iterator();

		InvoiceTo invoice;

		for (int i=0; iter.hasNext(); i++) {

			invoice = (InvoiceTo) iter.next();
			items.add(new TextItem(invoice.getId(), invoice.getInvoiceNumber() + "/" + invoice.getInvoiceSerial()));

		}

		return items;
	}
	
}
