/************************************************************************************************/
/*                                                                                              */
/*      <cyberHotel: Gestion Hotelera GPL>                                                      */
/*      Copyright (C) <2008> <davide05@gmail.com>                                               */
/*                                                                                              */
/*      This program is free software: you can redistribute it and/or modify                    */
/*      it under the terms of the GNU General Public License as published by                    */
/*      the Free Software Foundation, either version 3 of the License, or                       */
/*      (at your option) any later version.                                                     */
/*                                                                                              */
/*      This program is distributed in the hope that it will be useful,                         */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of                          */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                                    */ 
/*      See the GNU General Public License for more details.                                    */
/*                                                                                              */
/*      You should have received a copy of the GNU General Public License                       */
/*      along with this program.  If not, see <http://www.gnu.org/licenses/>.                   */
/*                                                                                              */
/************************************************************************************************/

package gz.cyberbrain.cyberhotel.controller;

import gz.cyberbrain.cyberhotel.model.bank.to.BankTo;
import gz.cyberbrain.cyberhotel.model.card.to.CardTo;
import gz.cyberbrain.cyberhotel.model.client.to.CompanyTo;
import gz.cyberbrain.cyberhotel.model.client.to.ParticularTo;
import gz.cyberbrain.cyberhotel.model.configurationfacade.delegate.ConfigurationFacadeDelegate;
import gz.cyberbrain.cyberhotel.model.reservationfacade.delegate.ReservationFacadeDelegate;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.ClientPayerException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.ClientWithInvoicesException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.InvalidDocumentTypeException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.InvalidPayModeException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.InvalidSexException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.ParticularWithOwesException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.ParticularWithReservationsException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableAddressException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableBankException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableCardException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableCompanyException;
import gz.cyberbrain.cyberhotel.model.spring.facade.SpringFacadeFactory;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InstanceNotFoundException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InternalErrorException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InvalidIntervalException;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.ImageItem;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.TextItem;

import java.io.Serializable;
import java.util.Calendar;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

public class ParticularController {

	private static ConfigurationFacadeDelegate config = SpringFacadeFactory.getConfigurationFacadeDelegate();
	private static ReservationFacadeDelegate delegate = SpringFacadeFactory.getReservationFacadeDelegate();
	private static HashMap<String, Object> attributes = new HashMap<String, Object>();

	private ParticularController() {}

	public static Vector particularList() {				
		Vector<TextItem> items = new Vector<TextItem>();
		Collection particulars;
		try {
			particulars = delegate.showParticulars();

			Iterator iter = particulars.iterator();

			ParticularTo particular;

			for (int i=0; iter.hasNext(); i++) {

				particular = (ParticularTo) iter.next();
				items.add(new TextItem(particular.getClientId(), particular.getFirstSurname() + " " + particular.getSecondSurname() + ", " + particular.getName()));

			}

		} catch (InternalErrorException e) {
			e.printStackTrace();
		}

		return items;
	}

	public static Serializable createParticular(Long address, String street, String dwellingPlace, 
			String phone1, String phone2, String fax, String email, String web, Long bank, Long card, String notes, 
			String name, String firstSurname, String secondSurname, Calendar bornDate, String documentType, 
			String documentNumber, Calendar expeditionDate, Calendar expirationDate, String sex, String nationality, 
			String job, Long company, String payMode, String accountNumber, String cardNumber, String voucherCode, 
			String clientType) throws InvalidSexException, InvalidDocumentTypeException, 
			InvalidPayModeException, UnaceptableAddressException, 
			UnaceptableCompanyException, UnaceptableBankException, UnaceptableCardException,
			InvalidIntervalException, InternalErrorException {

		return delegate.insertAParticular(new ParticularTo(null, address, street, dwellingPlace, phone1, phone2, fax, 
				email, web, bank, card, notes, name, firstSurname, secondSurname, bornDate, documentType, 
				documentNumber, expeditionDate, expirationDate, sex, nationality, job, company, payMode, accountNumber,
				cardNumber, voucherCode, clientType)).getClientId();

	}

	public static void updateParticular(Long clientId, Long address, String street, String dwellingPlace, 
			String phone1, String phone2, String fax, String email, String web, Long bank, Long card, String notes, 
			String name, String firstSurname, String secondSurname, Calendar bornDate, String documentType, 
			String documentNumber, Calendar expeditionDate, Calendar expirationDate, String sex, String nationality, 
			String job, Long company, String payMode, String accountNumber, String cardNumber, String voucherCode, 
			String clientType) throws InternalErrorException, InvalidSexException, 
			InvalidDocumentTypeException, InvalidPayModeException, UnaceptableAddressException, 
			UnaceptableCompanyException, UnaceptableBankException, UnaceptableCardException, 
			InvalidIntervalException, InstanceNotFoundException {				

		delegate.updateAParticular(new ParticularTo(clientId, address, street, dwellingPlace, phone1, phone2, fax, 
				email, web, bank, card, notes, name, firstSurname, secondSurname, bornDate, documentType, 
				documentNumber, expeditionDate, expirationDate, sex, nationality, job, company, payMode, accountNumber,
				cardNumber, voucherCode, clientType));

	}

	public static void deleteParticular(Long clientId) 
	throws InstanceNotFoundException, InternalErrorException, ParticularWithOwesException, 
	ParticularWithReservationsException, ClientPayerException, ClientWithInvoicesException {

		ParticularTo particular = delegate.findAParticular(clientId);
		delegate.removeAParticular(particular);

	}

	public static HashMap findParticular(Long clientId) throws InstanceNotFoundException, InternalErrorException {

		ParticularTo particular = delegate.findAParticular(clientId);

		attributes.put("id", clientId);
		attributes.put("address", particular.getAddress());
		attributes.put("street", particular.getStreet());
		attributes.put("dwellingPlace", particular.getDwellingPlace());
		attributes.put("phone1", particular.getPhone1());
		attributes.put("phone2", particular.getPhone2());
		attributes.put("fax", particular.getFax());
		attributes.put("email", particular.getEmail());
		attributes.put("web", particular.getWeb());

		BankTo bank = config.findBank(particular.getBank());
		attributes.put("bank", new ImageItem(bank.getBankId(), bank.getName(), bank.getLogo()));

		CardTo card = config.findCard(particular.getCard());
		attributes.put("card", new ImageItem(card.getCardId(), card.getType(), card.getLogo()));

		attributes.put("notes", particular.getNotes());
		attributes.put("name", particular.getName());
		attributes.put("firstSurname", particular.getFirstSurname());		
		attributes.put("secondSurname", particular.getSecondSurname());
		attributes.put("bornDate", particular.getBornDate());
		attributes.put("documentType", particular.getDocumentType());
		attributes.put("documentNumber", particular.getDocumentNumber());
		attributes.put("expeditionDate", particular.getExpeditionDate());
		attributes.put("expirationDate", particular.getExpirationDate());
		attributes.put("sex", particular.getSex());
		attributes.put("nationality", particular.getNationality());
		attributes.put("job", particular.getJob());

		if (particular.getCompany() != null) {
			CompanyTo company = delegate.findACompany(particular.getCompany());
			attributes.put("company", new TextItem(company.getClientId(), company.getName()));
		} else attributes.put("company", new TextItem(new Long(0), ""));

		attributes.put("payMode", particular.getPayMode());
		attributes.put("accountNumber", particular.getAccountNumber());
		attributes.put("cardNumber", particular.getCardNumber());
		attributes.put("voucherCode", particular.getVoucherCode());
		attributes.put("clientType", particular.getClientType());		

		return attributes;

	}

}
