/************************************************************************************************/
/*                                                                                              */
/*      <cyberHotel: Gestion Hotelera GPL>                                                      */
/*      Copyright (C) <2008> <davide05@gmail.com>                                               */
/*                                                                                              */
/*      This program is free software: you can redistribute it and/or modify                    */
/*      it under the terms of the GNU General Public License as published by                    */
/*      the Free Software Foundation, either version 3 of the License, or                       */
/*      (at your option) any later version.                                                     */
/*                                                                                              */
/*      This program is distributed in the hope that it will be useful,                         */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of                          */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                                    */ 
/*      See the GNU General Public License for more details.                                    */
/*                                                                                              */
/*      You should have received a copy of the GNU General Public License                       */
/*      along with this program.  If not, see <http://www.gnu.org/licenses/>.                   */
/*                                                                                              */
/************************************************************************************************/

package gz.cyberbrain.cyberhotel.model.configurationfacade.spring.actions;

import gz.cyberbrain.cyberhotel.model.board.dao.BoardDao;
import gz.cyberbrain.cyberhotel.model.configurationfacade.exceptions.UnaceptableBoardException;
import gz.cyberbrain.cyberhotel.model.configurationfacade.exceptions.UnaceptableSeasonException;
import gz.cyberbrain.cyberhotel.model.price.dao.PriceDao;
import gz.cyberbrain.cyberhotel.model.price.to.PriceTo;
import gz.cyberbrain.cyberhotel.model.pricetype.dao.PriceTypeDao;
import gz.cyberbrain.cyberhotel.model.configurationfacade.exceptions.UnaceptablePriceTypeException;
import gz.cyberbrain.cyberhotel.model.season.dao.SeasonDao;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InstanceNotFoundException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InternalErrorException;

public class CreatePriceAction {

	private PriceTo priceTo;
	private BoardDao boardDao;
	private PriceTypeDao priceTypeDao;
	private SeasonDao seasonDao;
	private PriceDao priceDao;
	
	public CreatePriceAction(PriceTo priceTo, BoardDao boardDao, PriceTypeDao priceTypeDao, SeasonDao seasonDao, PriceDao priceDao) {
		
		this.priceTo = priceTo;
		this.boardDao = boardDao;
		this.priceTypeDao = priceTypeDao;
		this.seasonDao = seasonDao;
		this.priceDao = priceDao;
		
	}
	
	public void execute() 
	throws UnaceptablePriceTypeException, UnaceptableBoardException, 
	UnaceptableSeasonException, InternalErrorException {
		
		try {
			boardDao.find(priceTo.getBoard());
		} catch (InstanceNotFoundException e) {
			throw new UnaceptableBoardException(priceTo.getBoard());
		}
		
		try {
			priceTypeDao.find(priceTo.getPriceType());
		} catch (InstanceNotFoundException e) {
			throw new UnaceptablePriceTypeException(priceTo.getPriceType());
		}
		
		try {
			seasonDao.find(priceTo.getSeason());
		} catch (InstanceNotFoundException e) {
			throw new UnaceptableSeasonException(priceTo.getSeason());
		}
		
		priceDao.create(priceTo);
		
	}

}
