/************************************************************************************************/
/*                                                                                              */
/*      <cyberHotel: Gestion Hotelera GPL>                                                      */
/*      Copyright (C) <2008> <davide05@gmail.com>                                               */
/*                                                                                              */
/*      This program is free software: you can redistribute it and/or modify                    */
/*      it under the terms of the GNU General Public License as published by                    */
/*      the Free Software Foundation, either version 3 of the License, or                       */
/*      (at your option) any later version.                                                     */
/*                                                                                              */
/*      This program is distributed in the hope that it will be useful,                         */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of                          */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                                    */ 
/*      See the GNU General Public License for more details.                                    */
/*                                                                                              */
/*      You should have received a copy of the GNU General Public License                       */
/*      along with this program.  If not, see <http://www.gnu.org/licenses/>.                   */
/*                                                                                              */
/************************************************************************************************/

package gz.cyberbrain.cyberhotel.model.invoicingfacade.spring.actions;

import gz.cyberbrain.cyberhotel.model.invoice.dao.InvoiceDao;
import gz.cyberbrain.cyberhotel.model.invoice.to.InvoiceTo;
import gz.cyberbrain.cyberhotel.model.invoicingfacade.exceptions.IssuedInvoiceException;
import gz.cyberbrain.cyberhotel.model.payment.dao.PaymentDao;
import gz.cyberbrain.cyberhotel.model.payment.to.PaymentTo;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableInvoiceException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InstanceNotFoundException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InternalErrorException;

public class InsertAPaymentAction {

	private InvoiceDao invoiceDao;
	private PaymentDao paymentDao;
	private PaymentTo paymentTo;
	boolean issue;
	
	public InsertAPaymentAction(InvoiceDao invoiceDao, PaymentDao paymentDao, PaymentTo paymentTo, boolean issue) {
		this.invoiceDao = invoiceDao;
		this.paymentDao = paymentDao;
		this.paymentTo = paymentTo;
		this.issue = issue;
	}
	
	public PaymentTo execute() throws UnaceptableInvoiceException, IssuedInvoiceException, InstanceNotFoundException, InternalErrorException {
		
		InvoiceTo invoice = null;
		
		try {
			invoice = invoiceDao.find(paymentTo.getInvoiceNumber(), paymentTo.getInvoiceSerial());	
		} catch (InstanceNotFoundException e) {
			throw new UnaceptableInvoiceException(paymentTo.getInvoiceNumber(), paymentTo.getInvoiceSerial());
		} catch (InternalErrorException e) {
			throw e;
		}
		
		//Invoice must be outstanding
		
		if (invoice.getStatus().equals("Emitida")) 
			throw new IssuedInvoiceException(invoice.getInvoiceNumber(), invoice.getInvoiceSerial());
		
		//Issue the invoice
		
		if (issue) invoice.setStatus("Emitida");
		
		invoiceDao.update(invoice);
		
		return paymentDao.create(paymentTo);
		
	}
	
}
