/************************************************************************************************/
/*                                                                                              */
/*      <cyberHotel: Gestion Hotelera GPL>                                                      */
/*      Copyright (C) <2008> <davide05@gmail.com>                                               */
/*                                                                                              */
/*      This program is free software: you can redistribute it and/or modify                    */
/*      it under the terms of the GNU General Public License as published by                    */
/*      the Free Software Foundation, either version 3 of the License, or                       */
/*      (at your option) any later version.                                                     */
/*                                                                                              */
/*      This program is distributed in the hope that it will be useful,                         */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of                          */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                                    */ 
/*      See the GNU General Public License for more details.                                    */
/*                                                                                              */
/*      You should have received a copy of the GNU General Public License                       */
/*      along with this program.  If not, see <http://www.gnu.org/licenses/>.                   */
/*                                                                                              */
/************************************************************************************************/

package gz.cyberbrain.cyberhotel.model.invoicingfacade.spring.actions;

import gz.cyberbrain.cyberhotel.model.client.dao.ClientDao;
import gz.cyberbrain.cyberhotel.model.invoice.dao.InvoiceDao;
import gz.cyberbrain.cyberhotel.model.invoice.to.InvoiceTo;
import gz.cyberbrain.cyberhotel.model.invoicingfacade.exceptions.UnaceptablePayerException;
import gz.cyberbrain.cyberhotel.model.owe.dao.OweDao;
import gz.cyberbrain.cyberhotel.model.owe.to.OweTo;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InstanceNotFoundException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InternalErrorException;

import java.util.Collection;
import java.util.Iterator;

public class UpdateAnInvoiceAction {

	private InvoiceDao invoiceDao;
	private OweDao oweDao;
	private ClientDao agencyDao;
	private ClientDao companyDao;
	private ClientDao particularDao;
	private InvoiceTo invoiceTo;
	private Collection owes;

	public UpdateAnInvoiceAction(InvoiceDao invoiceDao, OweDao oweDao, ClientDao agencyDao, ClientDao companyDao, ClientDao particularDao, InvoiceTo invoiceTo, Collection owes) {
		
		this.invoiceDao = invoiceDao;
		this.oweDao = oweDao;
		this.agencyDao = agencyDao;
		this.companyDao = companyDao;
		this.particularDao = particularDao;
		this.invoiceTo = invoiceTo;
		this.owes = owes;
		
	}


	public InvoiceTo execute() throws UnaceptablePayerException, InstanceNotFoundException, InternalErrorException {

		//Payer must exist
		
		Collection existentAgencies = agencyDao.findAllIds();
		Collection existentCompanies = companyDao.findAllIds();
		Collection existentParticulars = particularDao.findAllIds();
		
		Long currentPayerId = invoiceTo.getPayer();
		
		if (currentPayerId == null) throw new UnaceptablePayerException(currentPayerId);
		if(!existentParticulars.contains(currentPayerId)
				&& !existentAgencies.contains(currentPayerId)
				&& !existentCompanies.contains(currentPayerId)) throw new UnaceptablePayerException(currentPayerId);			
				
		//Deassociate previous owes from invoice
		
		OweTo owe;
		
		Collection oldOwes = invoiceDao.findInvoiceOwes(invoiceTo.getInvoiceNumber(), invoiceTo.getInvoiceSerial());
		Iterator iter = oldOwes.iterator();

		for (int i=0; iter.hasNext();i++) {

			owe = (OweTo) iter.next();
			owe.setInvoiceNumber(null);
			owe.setInvoiceSerial(null);
			owe.setStatus("Pendente");
			oweDao.update(owe);

		}
	
		//Insert the invoice		
		invoiceDao.update(invoiceTo);

		//Associate owes to invoice
		iter = owes.iterator();
	
		for (int i=0; iter.hasNext();i++) {

			owe = oweDao.find((Long) iter.next());
			owe.setInvoiceNumber(invoiceTo.getInvoiceNumber());
			owe.setInvoiceSerial(invoiceTo.getInvoiceSerial());
			owe.setStatus("Facturado");
			oweDao.update(owe);

		}
		
		return invoiceTo;

	}

}
