/************************************************************************************************/
/*                                                                                              */
/*      <cyberHotel: Gestion Hotelera GPL>                                                      */
/*      Copyright (C) <2008> <davide05@gmail.com>                                               */
/*                                                                                              */
/*      This program is free software: you can redistribute it and/or modify                    */
/*      it under the terms of the GNU General Public License as published by                    */
/*      the Free Software Foundation, either version 3 of the License, or                       */
/*      (at your option) any later version.                                                     */
/*                                                                                              */
/*      This program is distributed in the hope that it will be useful,                         */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of                          */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                                    */ 
/*      See the GNU General Public License for more details.                                    */
/*                                                                                              */
/*      You should have received a copy of the GNU General Public License                       */
/*      along with this program.  If not, see <http://www.gnu.org/licenses/>.                   */
/*                                                                                              */
/************************************************************************************************/

package gz.cyberbrain.cyberhotel.model.room.dao;

import java.util.Calendar;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;

import gz.cyberbrain.cyberhotel.model.util.exceptions.InstanceNotFoundException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InternalErrorException;
import gz.cyberbrain.cyberhotel.model.room.to.RoomTo;
import gz.cyberbrain.cyberhotel.model.spring.dao.AbstractSpringDao;

public class RoomSpringDao extends AbstractSpringDao implements RoomDao {

	public RoomSpringDao() {
		super(RoomTo.class);
	}	

	public RoomTo create(RoomTo roomTo) throws InternalErrorException {		

		return (RoomTo) super.save(roomTo);

	}

	public RoomTo find(String roomCode) throws InstanceNotFoundException, InternalErrorException {

		RoomTo roomTo = (RoomTo) super.find(roomCode);
		return roomTo;

	}

	public void update(RoomTo roomTo) throws InternalErrorException {		

		super.update(roomTo);

	}

	public void remove(RoomTo roomTo) throws InternalErrorException {

			super.delete(roomTo);
	
	}
	
	public Collection findAll() throws InternalErrorException {
		
		String queryName = "allRooms";
		return super.findByQuery(queryName);
		
	}
	
	public Collection findByType (String type) throws InternalErrorException{
		
		String queryName = "roomByType"; 
		
		return super.findByAttribute(queryName, type);
		
	}
	
	public Collection findByOccupancyStatus (String occupancyStatus) throws InternalErrorException {
		
		String queryName = "roomByOccupancyStatus";
		
		return super.findByAttribute(queryName, occupancyStatus);
		
	}
	
    public Collection findByCleaningStatus (String cleaningStatus) throws InternalErrorException {
		
		String queryName = "roomByCleaningStatus";
		
		return super.findByAttribute(queryName, cleaningStatus);
		
	}	
	
	public Collection findByTypeAndOccupancy (String []attributes) throws InternalErrorException {
		
		String queryName = "roomByTypeAndOccupancy";
		
		return super.findByAttributes(queryName, attributes);
		
	}
	
	public boolean existBoarders(String roomCode) throws InternalErrorException {
		
		String queryName = "roomBoarders";
		
		return (!super.findByAttribute(queryName, roomCode).isEmpty());
	}
	
	public Collection freeBetweenDates(Calendar []dates) 
		throws InternalErrorException {
		
		String queryName = "freeBetweenDates";
		return super.findByAttributes(queryName, dates);
	}
	
	public Collection findAllIds() throws InternalErrorException {

		String queryName = "allRoomIds";
		return super.findByQuery(queryName);
	}
	
	public Long findCurrentCheckIn(String roomCode) throws InternalErrorException {
		
		String queryName = "roomCheckIn";
		Collection checkIn = new HashSet <Long>();
		checkIn = super.findByAttribute(queryName, roomCode);
		Exception e = new Exception("Multiple room check-ins!!!");
		if (checkIn.size() > 1) throw new InternalErrorException(e);
		Iterator iter = checkIn.iterator();
		Integer reservationId = (Integer) iter.next();
		return reservationId.longValue();
		
	}
	
}
