/************************************************************************************************/
/*                                                                                              */
/*      <cyberHotel: Gestion Hotelera GPL>                                                      */
/*      Copyright (C) <2008> <davide05@gmail.com>                                               */
/*                                                                                              */
/*      This program is free software: you can redistribute it and/or modify                    */
/*      it under the terms of the GNU General Public License as published by                    */
/*      the Free Software Foundation, either version 3 of the License, or                       */
/*      (at your option) any later version.                                                     */
/*                                                                                              */
/*      This program is distributed in the hope that it will be useful,                         */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of                          */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                                    */ 
/*      See the GNU General Public License for more details.                                    */
/*                                                                                              */
/*      You should have received a copy of the GNU General Public License                       */
/*      along with this program.  If not, see <http://www.gnu.org/licenses/>.                   */
/*                                                                                              */
/************************************************************************************************/

package gz.cyberbrain.cyberhotel.model.spring.dao;

import gz.cyberbrain.cyberhotel.model.identifiable.Identifiable;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InstanceNotFoundException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InternalErrorException;

import java.io.Serializable;
import java.util.Collection;

import org.hibernate.ObjectDeletedException;
import org.springframework.dao.DataAccessException;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;

public class AbstractSpringDao extends HibernateDaoSupport {

	private Class beanClass;

	public AbstractSpringDao(Class beanClass){
		this.beanClass = beanClass;
	}

	public AbstractSpringDao(){}

	protected Identifiable save(Identifiable obj) throws InternalErrorException {

		try{

			getHibernateTemplate().save(obj);
			return obj;

		}catch(DataAccessException e) {
			throw new InternalErrorException(e);
		}
	}

	protected void update(Identifiable obj) throws InternalErrorException {

		try{

			getHibernateTemplate().update(obj);

		} catch (DataAccessException e) {
			throw new InternalErrorException(e);
		}

	}
	
	protected void saveOrUpdate(Identifiable obj) throws InternalErrorException {

		try{

			getHibernateTemplate().saveOrUpdate(obj);

		} catch (DataAccessException e) {
			throw new InternalErrorException(e);
		}

	}

	protected void delete(Identifiable obj) throws InternalErrorException {

		try{

			getHibernateTemplate().delete(obj);		

		}catch (DataAccessException e) {
			throw new InternalErrorException(e);
		} 

	}
	
	protected void deleteAll(Collection objs) throws InternalErrorException {

		try{

			getHibernateTemplate().deleteAll(objs);		

		}catch (DataAccessException e) {
			throw new InternalErrorException(e);
		} 

	}

	protected Identifiable find(Serializable id) throws InstanceNotFoundException, InternalErrorException {

		try{

			Identifiable identifiable = 
				(Identifiable) getHibernateTemplate().get(beanClass, id); 

			if (identifiable == null)	
				throw new InstanceNotFoundException(id, beanClass.getName());
			
			return identifiable; 

		} catch (DataAccessException e) {
			

			if (e.contains(ObjectDeletedException.class)) 
				throw new InstanceNotFoundException(id, beanClass.getName());
			

			throw new InternalErrorException(e);
		}

	}

	protected Collection findByQuery(String queryName) throws InternalErrorException {

		try {

			return getHibernateTemplate().findByNamedQuery(queryName);

		} catch (DataAccessException e) {
			throw new InternalErrorException(e);
		}
	}

	protected Collection findByAttributes(String queryName, Serializable [] attributes) throws InternalErrorException {

		try {

			return getHibernateTemplate().findByNamedQuery(queryName, attributes);

		}catch (DataAccessException e) {
			throw new InternalErrorException(e);
		}
	}

	protected Collection findByAttribute(String queryName, Serializable attribute) throws InternalErrorException {

		try {

			return getHibernateTemplate().findByNamedQuery(queryName, attribute);

		}catch (DataAccessException e) {
			throw new InternalErrorException(e);
		}
	}

}
