/************************************************************************************************/
/*                                                                                              */
/*      <cyberHotel: Gestion Hotelera GPL>                                                      */
/*      Copyright (C) <2008> <davide05@gmail.com>                                               */
/*                                                                                              */
/*      This program is free software: you can redistribute it and/or modify                    */
/*      it under the terms of the GNU General Public License as published by                    */
/*      the Free Software Foundation, either version 3 of the License, or                       */
/*      (at your option) any later version.                                                     */
/*                                                                                              */
/*      This program is distributed in the hope that it will be useful,                         */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of                          */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                                    */ 
/*      See the GNU General Public License for more details.                                    */
/*                                                                                              */
/*      You should have received a copy of the GNU General Public License                       */
/*      along with this program.  If not, see <http://www.gnu.org/licenses/>.                   */
/*                                                                                              */
/************************************************************************************************/

package gz.cyberbrain.cyberhotel.view.swing.invoicing;

import gz.cyberbrain.cyberhotel.controller.CashListAdapter;
import gz.cyberbrain.cyberhotel.controller.OpenedPaymentListAdapter;
import gz.cyberbrain.cyberhotel.controller.SaveAdapter;
import gz.cyberbrain.cyberhotel.controller.util.ErrorsHandler;
import gz.cyberbrain.cyberhotel.model.util.dates.DateOperations;
import gz.cyberbrain.cyberhotel.model.util.money.MoneyOperations;
import gz.cyberbrain.cyberhotel.security.SecurityHandler;
import gz.cyberbrain.cyberhotel.view.swing.ToolsWindow;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.DateFormat;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.InfoPanel;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.Payment;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.ToolsPanel;
import gz.cyberbrain.cyberhotel.view.swing.util.locale.Locale;

import java.awt.Color;
import java.awt.Font;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.KeyEvent;
import java.io.Serializable;
import java.math.BigDecimal;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.HashMap;
import java.util.Vector;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.text.Position;

public class CashWindow extends ToolsWindow {

	private static final long serialVersionUID = 1L;
	private static CashWindow cashWindow = null;
	private Serializable id = null;
	private SaveAdapter saveAdapter = null;
	private String dialogMessage = null;
	private String dialogTitle = null;
	private InfoPanel infoPanel = null;
	private JPanel jContentPane = null;
	private JPanel firstPanel = null;
	private JLabel searchLabel = null;
	private JTextField searchTextField = null;
	private JScrollPane listScrollPane = null;
	private JList paymentsList = null;
	private JButton cashButton = null;
	private JPanel secondPanel = null;
	private JLabel numLabel = null;
	private JTextField numTextField = null;
	private JLabel openDateLabel = null;
	private JFormattedTextField openDateTextField = null;
	private JLabel closeDateLabel = null;
	private JFormattedTextField closeDateTextField = null;
	private JLabel openUserLabel = null;
	private JTextField openUserTextField = null;
	private JLabel closeUserLabel = null;
	private JTextField closeUserTextField = null;
	private JPanel cashPanel = null;
	private JLabel paymentsTextLabel = null;
	private JLabel accountsTextLabel = null;
	private JLabel cardsTextLabel = null;
	private JLabel vouchersTextLabel = null;
	private JLabel totalTextLabel = null;
	private JButton closeButton = null;
	private JButton saveButton = null;
	private JButton cancelButton = null;
	private ToolsPanel toolsPanel = null;
	private JLabel statusLabel = null;
	private JComboBox statusComboBox = null;
	private Object[] payments = new Object[0];
	private JLabel accountsAmountTextLabel = null;
	private JLabel cardsAmountTextLabel = null;
	private JLabel vouchersAmountTextLabel = null;
	private JLabel paymentsLabel = null;
	private JLabel accountsLabel = null;
	private JLabel cardsLabel = null;
	private JLabel vouchersLabel = null;
	private JLabel accountsAmountLabel = null;
	private JLabel cardsAmountLabel = null;
	private JLabel vouchersAmountLabel = null;
	private JLabel totalLabel = null;

	private long paymentsNumber = 0;
	private long accountsNumber = 0;
	private long cardsNumber = 0;
	private long vouchersNumber = 0;
	private double accountsTotal = 0;
	private double cardsTotal = 0;
	private double vouchersTotal = 0;
	private double cashTotal = 0;

	/**
	 * This is the xxx default constructor
	 */
	private CashWindow() {
		super("", false, true, false, true);
		initialize();
	}

	public static CashWindow getInstance() {
		if (cashWindow == null) cashWindow = new CashWindow();
		return cashWindow;
	}

	public void setAdapter(SaveAdapter saveAdapter) {
		this.saveAdapter = saveAdapter;
	}

	public SaveAdapter getAdapter() {
		return saveAdapter;
	}

	public void setId(Serializable id) {
		this.id = id;
	}

	public Serializable getId() {
		return id;
	}

	public String getDialogMessage() {
		return dialogMessage;
	}

	public void setDialogMessage(String dialogMessage) {
		this.dialogMessage = dialogMessage;
	}

	public String getDialogTitle() {
		return dialogTitle;
	}

	public void setDialogTitle(String dialogTitle) {
		this.dialogTitle = dialogTitle;
	}

	private Object[] getPayments() {
		return payments;
	}

	private void setPayments(Object[] payments) {
		this.payments = payments;
	}
	/**
	 * This method initializes infoPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private InfoPanel getInfoPanel() {
		if (infoPanel == null) {
			infoPanel = new InfoPanel();
			infoPanel.setBounds(new Rectangle(0, 0, this.getWidth(), 50));
		}
		return infoPanel;
	}

	public void setInfoPanel(InfoPanel ipanel) {		  

		infoPanel = ipanel;
		jContentPane.add(infoPanel);

	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(683, 520);
		this.setContentPane(getJContentPane());
		this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		this.addInternalFrameListener(new javax.swing.event.InternalFrameAdapter()
		{

			public void internalFrameClosed(javax.swing.event.InternalFrameEvent evt)
			{
				(CashWindow.getInstance()).clean();
				(CashWindow.getInstance()).lock();
				getToolsPanel().reset();
				cashWindow = null;
			}

		});
		lock();
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(null);
			jContentPane.add(getFirstPanel(), null);
			jContentPane.add(getSecondPanel(), null);
			jContentPane.add(getInfoPanel(), null);			
		}
		return jContentPane;
	}

	/**
	 * This method initializes firstPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getFirstPanel() {
		if (firstPanel == null) {
			searchLabel = new JLabel();
			searchLabel.setBounds(new Rectangle(7, 11, 38, 16));
			searchLabel.setText(Locale.get("CashWindow.searchLabel"));
			firstPanel = new JPanel();
			firstPanel.setLayout(null);
			firstPanel.setBounds(new Rectangle(0, 51, 220, 434));
			firstPanel.add(searchLabel, null);
			firstPanel.add(getSearchTextField(), null);
			firstPanel.add(getListScrollPane(), null);
			firstPanel.add(getCashButton(), null);
		}
		return firstPanel;
	}

	/**
	 * This method initializes searchTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getSearchTextField() {
		if (searchTextField == null) {
			searchTextField = new JTextField();
			searchTextField.setBounds(new Rectangle(55, 11, 157, 20));
			searchTextField.addKeyListener(new java.awt.event.KeyAdapter() {
				public void keyPressed(java.awt.event.KeyEvent e) {

					String text = searchTextField.getText(); 

					try {

						if (e.getKeyCode() == KeyEvent.VK_BACK_SPACE && text.length() != 0) getPaymentsList().setSelectedIndex(getPaymentsList().getNextMatch(text.substring(0,text.length() - 1), 0, Position.Bias.Forward));
						else getPaymentsList().setSelectedIndex(getPaymentsList().getNextMatch(text + e.getKeyChar(), 0, Position.Bias.Forward));

					} catch (IllegalArgumentException f) {} 

				}
			});
		}
		return searchTextField;
	}

	/**
	 * This method initializes listScrollPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getListScrollPane() {
		if (listScrollPane == null) {
			listScrollPane = new JScrollPane();
			listScrollPane.setBounds(new Rectangle(5, 36, 207, 287));
			listScrollPane.setViewportView(getPaymentsList());
		}
		return listScrollPane;
	}

	private void loadList() {
		DefaultListModel model = new DefaultListModel();						
		Vector data = (new OpenedPaymentListAdapter()).load();
		for (int i=0; i < data.size(); i++) {
			model.add(i, data.elementAt(i));
		}
		paymentsList.setModel(model);
		if (paymentsList.getModel().getSize() != 0) paymentsList.setSelectedIndex(0);

	}

	/**
	 * This method initializes paymentsList	
	 * 	
	 * @return javax.swing.JList	
	 */
	private JList getPaymentsList() {
		if (paymentsList == null) {
			paymentsList = new JList();
			//paymentsList.setEnabled(false);
			loadList();
		}
		return paymentsList;
	}

	/**
	 * This method initializes cashButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getCashButton() {
		if (cashButton == null) {
			cashButton = new JButton();
			cashButton.setBounds(new Rectangle(138, 342, 74, 17));
			cashButton.setText(Locale.get("CashWindow.cashButton"));
			cashButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {

					if (getPaymentsList().getSelectedValues().length != 0) 
						setPayments(getPaymentsList().getSelectedValues());

					Payment payment;
					
					long payments = new Long(paymentsLabel.getText()).longValue();
					long accounts = new Long(accountsLabel.getText()).longValue();
					long cards =  new Long(cardsLabel.getText()).longValue();
					long vouchers =  new Long(vouchersLabel.getText()).longValue();
					double accountsAmount =  new Double(accountsAmountLabel.getText()).doubleValue();
					double cardsAmount =  new Double(cardsAmountLabel.getText()).doubleValue();
					double vouchersAmount =  new Double(vouchersAmountLabel.getText()).doubleValue();
					double total =  new Double(totalLabel.getText()).doubleValue();

					for (int i=0; i<getPayments().length; i++) {

						payment = (Payment) getPayments()[i];

						if (payment.getPayMode().equals("Contado")) {
							accounts += 1;
							accountsAmount += payment.getAmount().doubleValue();
						}
						if (payment.getPayMode().equals("Tarxeta")) {
							cards  += 1;
							cardsAmount += payment.getAmount().doubleValue();
						}
						if (payment.getPayMode().equals("Bono")) {
							vouchers += 1;
							vouchersAmount += payment.getAmount().doubleValue();
						}

					}

					payments += getPayments().length;
					
					paymentsLabel.setText("" + payments);
					accountsLabel.setText("" + accounts);
					cardsLabel.setText("" + cards);
					vouchersLabel.setText("" + vouchers);

					accountsAmountLabel.setText("" + accountsAmount);
					cardsAmountLabel.setText("" + cardsAmount);
					vouchersAmountLabel.setText("" + vouchersAmount);

					total = accountsAmount + cardsAmount + vouchersAmount;

					totalLabel.setText("" + MoneyOperations.getDecimalReduction(new BigDecimal(total)));

					getSaveButton().setEnabled(true);
					getCancelButton().setEnabled(true);

					cashButton.setEnabled(false);

				}
			});
		}
		return cashButton;
	}

	/**
	 * This method initializes secondPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getSecondPanel() {
		if (secondPanel == null) {
			statusLabel = new JLabel();
			statusLabel.setBounds(new Rectangle(16, 41, 38, 16));
			statusLabel.setText(Locale.get("CashWindow.statusLabel"));
			closeUserLabel = new JLabel();
			closeUserLabel.setBounds(new Rectangle(285, 41, 67, 16));
			closeUserLabel.setText(Locale.get("CashWindow.closeUserLabel"));
			openUserLabel = new JLabel();
			openUserLabel.setBounds(new Rectangle(143, 41, 60, 16));
			openUserLabel.setText(Locale.get("CashWindow.openUserLabel"));
			closeDateLabel = new JLabel();
			closeDateLabel.setBounds(new Rectangle(285, 13, 67, 16));
			closeDateLabel.setText(Locale.get("CashWindow.closeDateLabel"));
			openDateLabel = new JLabel();
			openDateLabel.setBounds(new Rectangle(143, 13, 60, 16));
			openDateLabel.setText(Locale.get("CashWindow.openDateLabel"));
			numLabel = new JLabel();
			numLabel.setBounds(new Rectangle(16, 13, 38, 16));
			numLabel.setText(Locale.get("CashWindow.numLabel"));
			secondPanel = new JPanel();
			secondPanel.setLayout(null);
			secondPanel.setBounds(new Rectangle(221, 51, 450, 434));
			secondPanel.add(numLabel, null);
			secondPanel.add(getNumTextField(), null);
			secondPanel.add(openDateLabel, null);
			secondPanel.add(getOpenDateTextField(), null);
			secondPanel.add(closeDateLabel, null);
			secondPanel.add(getCloseDateTextField(), null);
			secondPanel.add(openUserLabel, null);
			secondPanel.add(getOpenUserTextField(), null);
			secondPanel.add(closeUserLabel, null);
			secondPanel.add(getCloseUserTextField(), null);
			secondPanel.add(getCashPanel(), null);
			secondPanel.add(getCloseButton(), null);
			secondPanel.add(getSaveButton(), null);
			secondPanel.add(getCancelButton(), null);
			secondPanel.add(getToolsPanel(), null);
			secondPanel.add(statusLabel, null);
			secondPanel.add(getStatusComboBox(), null);
		}
		return secondPanel;
	}

	/**
	 * This method initializes numTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getNumTextField() {
		if (numTextField == null) {
			numTextField = new JTextField();
			numTextField.setBounds(new Rectangle(57, 12, 82, 20));
			numTextField.setEnabled(false);
		}
		return numTextField;
	}

	/**
	 * This method initializes openDateTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JFormattedTextField getOpenDateTextField() {
		if (openDateTextField == null) {
			try {
				openDateTextField = new JFormattedTextField(new DateFormat());
			} catch (ParseException e) {
				e.printStackTrace();
			}
			openDateTextField.setBounds(new Rectangle(207, 12, 74, 20));
			openDateTextField.setEditable(false);
		}
		return openDateTextField;
	}

	/**
	 * This method initializes closeDateTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JFormattedTextField getCloseDateTextField() {
		if (closeDateTextField == null) {
			try {
				closeDateTextField = new JFormattedTextField(new DateFormat());
			} catch (ParseException e) {
				e.printStackTrace();
			}
			closeDateTextField.setBounds(new Rectangle(356, 12, 74, 20));
			closeDateTextField.setEditable(false);
		}
		return closeDateTextField;
	}

	/**
	 * This method initializes statusComboBox	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	private JComboBox getStatusComboBox() {
		if (statusComboBox == null) {
			statusComboBox = new JComboBox();
			statusComboBox.setBounds(new Rectangle(57, 40, 82, 20));
			statusComboBox.addItem(Locale.get("CashWindow.statusComboBox.item0"));
			statusComboBox.addItem(Locale.get("CashWindow.statusComboBox.item1"));	
		}
		return statusComboBox;
	}

	/**
	 * This method initializes openUserTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getOpenUserTextField() {
		if (openUserTextField == null) {
			openUserTextField = new JTextField();
			openUserTextField.setBounds(new Rectangle(207, 40, 74, 20));
			openUserTextField.setEditable(false);
		}
		return openUserTextField;
	}

	/**
	 * This method initializes closeUserTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getCloseUserTextField() {
		if (closeUserTextField == null) {
			closeUserTextField = new JTextField();
			closeUserTextField.setBounds(new Rectangle(356, 40, 74, 20));
			closeUserTextField.setEditable(false);
		}
		return closeUserTextField;
	}

	/**
	 * This method initializes cashPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getCashPanel() {
		if (cashPanel == null) {
			totalLabel = new JLabel();
			totalLabel.setBounds(new Rectangle(115, 224, 196, 16));
			totalLabel.setFont(new Font(totalLabel.getFont().getName(), Font.BOLD, totalLabel.getFont().getSize()));
			totalLabel.setText("0");
			vouchersAmountLabel = new JLabel();
			vouchersAmountLabel.setBounds(new Rectangle(115, 176, 196, 16));
			vouchersAmountLabel.setText("0");
			cardsAmountLabel = new JLabel();
			cardsAmountLabel.setBounds(new Rectangle(115, 154, 196, 16));
			cardsAmountLabel.setText("0");
			accountsAmountLabel = new JLabel();
			accountsAmountLabel.setBounds(new Rectangle(115, 132, 196, 16));
			accountsAmountLabel.setText("0");
			vouchersLabel = new JLabel();
			vouchersLabel.setBounds(new Rectangle(115, 93, 196, 16));
			vouchersLabel.setText("0");
			vouchersLabel.setForeground(Color.GRAY);
			cardsLabel = new JLabel();
			cardsLabel.setBounds(new Rectangle(115, 71, 196, 16));
			cardsLabel.setText("0");
			cardsLabel.setForeground(Color.GRAY);
			accountsLabel = new JLabel();
			accountsLabel.setBounds(new Rectangle(115, 49, 196, 16));
			accountsLabel.setText("0");
			accountsLabel.setForeground(Color.GRAY);
			paymentsLabel = new JLabel();
			paymentsLabel.setBounds(new Rectangle(115, 11, 196, 16));
			paymentsLabel.setText("0");
			paymentsLabel.setFont(new Font(paymentsLabel.getFont().getName(), Font.BOLD, paymentsLabel.getFont().getSize()));
			paymentsLabel.setForeground(Color.BLUE);
			vouchersAmountTextLabel = new JLabel();
			vouchersAmountTextLabel.setBounds(new Rectangle(10, 176, 88, 16));
			vouchersAmountTextLabel.setText(Locale.get("CashWindow.vouchersAmountTextLabel"));
			cardsAmountTextLabel = new JLabel();
			cardsAmountTextLabel.setBounds(new Rectangle(10, 154, 88, 16));
			cardsAmountTextLabel.setText(Locale.get("CashWindow.cardsAmountTextLabel"));
			accountsAmountTextLabel = new JLabel();
			accountsAmountTextLabel.setBounds(new Rectangle(10, 132, 88, 16));
			accountsAmountTextLabel.setText(Locale.get("CashWindow.accountsAmountTextLabel"));
			totalTextLabel = new JLabel();
			totalTextLabel.setBounds(new Rectangle(10, 224, 88, 16));
			totalTextLabel.setText(Locale.get("CashWindow.totalTextLabel"));
			totalTextLabel.setFont(new Font(totalTextLabel.getFont().getName(), Font.BOLD, totalTextLabel.getFont().getSize()));
			vouchersTextLabel = new JLabel();
			vouchersTextLabel.setBounds(new Rectangle(10, 93, 88, 16));
			vouchersTextLabel.setText(Locale.get("CashWindow.vouchersTextLabel"));
			vouchersTextLabel.setForeground(Color.GRAY);
			cardsTextLabel = new JLabel();
			cardsTextLabel.setBounds(new Rectangle(10, 71, 88, 16));
			cardsTextLabel.setText(Locale.get("CashWindow.cardsTextLabel"));
			cardsTextLabel.setForeground(Color.GRAY);
			accountsTextLabel = new JLabel();
			accountsTextLabel.setBounds(new Rectangle(10, 49, 88, 16));
			accountsTextLabel.setText(Locale.get("CashWindow.accountsTextLabel"));
			accountsTextLabel.setForeground(Color.GRAY);
			paymentsTextLabel = new JLabel();
			paymentsTextLabel.setBounds(new Rectangle(10, 11, 88, 16));
			paymentsTextLabel.setText(Locale.get("CashWindow.paymentsTextLabel"));
			paymentsTextLabel.setFont(new Font(paymentsTextLabel.getFont().getName(), Font.BOLD, paymentsTextLabel.getFont().getSize()));
			paymentsTextLabel.setForeground(Color.BLUE);
			cashPanel = new JPanel();
			cashPanel.setLayout(null);
			cashPanel.setBounds(new Rectangle(35, 71, 373, 250));
			cashPanel.add(paymentsTextLabel, null);
			cashPanel.add(accountsTextLabel, null);
			cashPanel.add(cardsTextLabel, null);
			cashPanel.add(vouchersTextLabel, null);
			cashPanel.add(totalTextLabel, null);
			cashPanel.setBackground(Color.WHITE);
			cashPanel.add(accountsAmountTextLabel, null);
			cashPanel.add(cardsAmountTextLabel, null);
			cashPanel.add(vouchersAmountTextLabel, null);
			cashPanel.add(paymentsLabel, null);
			cashPanel.add(accountsLabel, null);
			cashPanel.add(cardsLabel, null);
			cashPanel.add(vouchersLabel, null);
			cashPanel.add(accountsAmountLabel, null);
			cashPanel.add(cardsAmountLabel, null);
			cashPanel.add(vouchersAmountLabel, null);
			cashPanel.add(totalLabel, null);
		}
		return cashPanel;
	}

	/**
	 * This method initializes closeButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getCloseButton() {
		if (closeButton == null) {
			closeButton = new JButton();
			closeButton.setBounds(new Rectangle(35, 342, 76, 17));
			closeButton.setText(Locale.get("CashWindow.closeButton"));
			closeButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {

					HashMap<String, Object> attrs = new HashMap<String, Object>();					

					attrs.put("cashNumber", (Long) getId());

					attrs.put("openDate", DateOperations.textToCalendar(getOpenDateTextField().getText()));

					attrs.put("closeDate", Calendar.getInstance());

					attrs.put("status", "Pechada");

					attrs.put("openUser", getOpenUserTextField().getText());

					attrs.put("closeUser", SecurityHandler.getAuthenticatedLogin());

					//Payments status won't be changeable

					boolean save = false;

					if (getId() != null) { 

						setDialogMessage(Locale.get("JOptionPane.CashWindow.closeButton.questionMessage"));
						setDialogTitle(Locale.get("JOptionPane.CashWindow.closeButton.questionTitle"));

						Object[] options = {Locale.get("JOptionPane.Yes"), Locale.get("JOptionPane.No")};
						int n = JOptionPane.showOptionDialog(InvoiceWindow.getInstance(),
								getDialogMessage(),
								getDialogTitle(),
								JOptionPane.YES_NO_OPTION,
								JOptionPane.INFORMATION_MESSAGE,
								null,
								options,
								options[0]);

						if (n == JOptionPane.YES_OPTION) {
							save = true;

							//Pass the payments

							Collection <Long> references = new ArrayList <Long> ();		

							for (int i=0; i<getPayments().length;i++) references.add(((Payment)getPayments()[i]).getReference()); 

							attrs.put("payments", references);
						}

					}

					attrs.put("closement", true);
					
					//If user wants to save
					if (save) {

						setId(getAdapter().save(attrs));

						if (!inputErrors()) {										

							setDialogMessage(Locale.get("JOptionPane.CashWindow.closeButton.message"));
							setDialogTitle(Locale.get("JOptionPane.CashWindow.closeButton.title"));

							Object[] options = {Locale.get("JOptionPane.Ok")};
							int n = JOptionPane.showOptionDialog(InvoiceWindow.getInstance(),
									getDialogMessage(),
									getDialogTitle(),
									JOptionPane.OK_OPTION,
									JOptionPane.INFORMATION_MESSAGE,
									null,
									options,
									options[0]);

							if (n == JOptionPane.OK_OPTION) {							
								clean();
								update();
								lock();
								getToolsPanel().reset();					
							}																	

						}				

					}		

				}

			});
		}
		return closeButton;
	}

	/**
	 * This method initializes saveButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getSaveButton() {
		if (saveButton == null) {
			saveButton = new JButton();
			saveButton.setBounds(new Rectangle(234, 341, 82, 17));
			saveButton.setText(Locale.get("CashWindow.saveButton"));
			saveButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {

					HashMap<String, Object> attrs = new HashMap<String, Object>();					

					//Update
					if (getId() != null) attrs.put("cashNumber", (Long) getId());

					//Add
					if (getId() == null) attrs.put("openDate", Calendar.getInstance());
					else attrs.put("openDate", DateOperations.textToCalendar(getOpenDateTextField().getText()));


					attrs.put("closeDate", null);

					String status = null;

					switch (getStatusComboBox().getSelectedIndex()) {
					case 0:
						status = "Aberta";
						break;
					case 1:
						status = "Pechada";
						break;					
					}

					attrs.put("status", status);

					if (getId() == null) attrs.put("openUser", SecurityHandler.getAuthenticatedLogin());
					else attrs.put("openUser", getOpenUserTextField().getText());

					attrs.put("closeUser", null);

					//Payments status won't be changeable

					boolean save = false;

					//Update
					if (getId() != null) { 

						setDialogMessage(Locale.get("JOptionPane.CashWindow.saveButton.paymentsMessage"));
						setDialogTitle(Locale.get("JOptionPane.CashWindow.saveButton.paymentsTitle"));

						Object[] options = {Locale.get("JOptionPane.Yes"), Locale.get("JOptionPane.No")};
						int n = JOptionPane.showOptionDialog(InvoiceWindow.getInstance(),
								getDialogMessage(),
								getDialogTitle(),
								JOptionPane.YES_NO_OPTION,
								JOptionPane.INFORMATION_MESSAGE,
								null,
								options,
								options[0]);

						if (n == JOptionPane.YES_OPTION) {
							save = true;

							//Pass the payments

							Collection <Long> references = new ArrayList <Long> ();		

							for (int i=0; i<getPayments().length;i++) references.add(((Payment)getPayments()[i]).getReference()); 

							attrs.put("payments", references);
						}

					}

					attrs.put("closement", false);
					
					//If is an Add or is an Edit and user wants to save
					if ((getId() == null) || ((getId() != null) && save)) {

						setId(getAdapter().save(attrs));

						if (!inputErrors()) {										

							setDialogMessage(Locale.get("JOptionPane.CashWindow.saveButton.message"));
							setDialogTitle(Locale.get("JOptionPane.CashWindow.saveButton.title"));

							Object[] options = {Locale.get("JOptionPane.Ok")};
							int n = JOptionPane.showOptionDialog(InvoiceWindow.getInstance(),
									getDialogMessage(),
									getDialogTitle(),
									JOptionPane.OK_OPTION,
									JOptionPane.INFORMATION_MESSAGE,
									null,
									options,
									options[0]);

							if (n == JOptionPane.OK_OPTION) {							
								clean();
								update();
								lock();
								getToolsPanel().reset();					
							}																	

						}	

					}

				}

			});
		}
		return saveButton;
	}

	/**
	 * This method initializes cancelButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton();
			cancelButton.setBounds(new Rectangle(326, 341, 82, 17));
			cancelButton.setText(Locale.get("CashWindow.cancelButton"));
			cancelButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {

					(CashWindow.getInstance()).lock();
					getPaymentsList().setEnabled(true);
					getSearchTextField().setEditable(true);
					getCashButton().setEnabled(true);					

					loadList();

					getCloseButton().setEnabled(true);

					resetCash();

				}
			});
		}
		return cancelButton;
	}

	private void resetCash() {

		paymentsLabel.setText("" + paymentsNumber);
		accountsLabel.setText("" + accountsNumber);
		cardsLabel.setText("" + cardsNumber);
		vouchersLabel.setText("" + vouchersNumber);
		accountsAmountLabel.setText("" + accountsTotal);
		cardsAmountLabel.setText("" + cardsTotal);
		vouchersAmountLabel.setText("" + vouchersTotal);
		totalLabel.setText("" + cashTotal);

	}

	/**
	 * This method initializes toolsPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private ToolsPanel getToolsPanel() {
		if (toolsPanel == null) {
			toolsPanel = new ToolsPanel(this, new CashListAdapter());
			toolsPanel.setBounds(new Rectangle(13, 371, this.getWidth(), 50));
		}
		return toolsPanel;
	}	

	@Override
	public void cleanErrors() {

		getInfoPanel().setInfoText(getInfoPanel().getInfoText());

	}

	@Override
	public void lock() {
		getSaveButton().setEnabled(false);
		getCancelButton().setEnabled(false);
		getCashButton().setEnabled(false);
		getPaymentsList().setEnabled(false);
		getSearchTextField().setEditable(false);
		getCloseButton().setEnabled(false);
		getStatusComboBox().setEnabled(false);
	}

	@Override
	public boolean showErrors() {
		return inputErrors();
	}

	@Override
	public void unlock() {
		if (getId() == null) {
			//Add			
			getSaveButton().setEnabled(true);
			getCloseButton().setEnabled(false);
		} else {
			//Edit			
			
			//Closed cash can't be editable
			if (getStatusComboBox().getSelectedIndex() == 1) {
				
				getInfoPanel().setErrorText("<html>" + Locale.get("InfoPanel.errors.title") + "<br>" 
						+ Locale.get("CashWindow.errors.closedCash")+ "</html>");

				Toolkit.getDefaultToolkit().beep();
				
				throw new UnsupportedOperationException();
				
			}
			
			getSaveButton().setEnabled(true);
			getCashButton().setEnabled(true);
			getPaymentsList().setEnabled(true);
			getSearchTextField().setEditable(true);
			getCloseButton().setEnabled(true);
		}
	}

	@Override
	public void update() {
		loadList();
	}

	@Override
	public void clean() {
		setId(null);

		setPayments(new Object[0]);

		getSearchTextField().setText("");
		if (getPaymentsList().getModel().getSize() != 0) getPaymentsList().setSelectedIndex(0);

		getNumTextField().setText("");
		getOpenDateTextField().setText(DateOperations.calendarToText(Calendar.getInstance()));
		getCloseDateTextField().setText("");

		getStatusComboBox().setSelectedIndex(0);

		getOpenUserTextField().setText(SecurityHandler.getAuthenticatedLogin());
		getCloseUserTextField().setText("");

		paymentsLabel.setText("0");
		accountsLabel.setText("0");
		cardsLabel.setText("0");
		vouchersLabel.setText("0");
		accountsAmountLabel.setText("0");
		cardsAmountLabel.setText("0");
		vouchersAmountLabel.setText("0");
		totalLabel.setText("0");
	}

	@Override
	public void fill(HashMap fields) {
		
		setId((Long) fields.get("id"));
		getNumTextField().setText(((Long) fields.get("cashNumber")).toString());		
		getOpenDateTextField().setText(DateOperations.calendarToText((Calendar) fields.get("openDate")));

		Calendar closeDate = (Calendar) fields.get("closeDate");

		if (closeDate != null) getCloseDateTextField().setText(DateOperations.calendarToText(closeDate));
		else getCloseDateTextField().setText("");

		if (((String) fields.get("status")).equals("Aberta")) getStatusComboBox().setSelectedIndex(0);
		if (((String) fields.get("status")).equals("Pechada")) getStatusComboBox().setSelectedIndex(1);			

		getOpenUserTextField().setText((String) fields.get("openUser"));

		String closeUser = (String) fields.get("closeUser");

		if (closeUser != null) getCloseUserTextField().setText(closeUser);
		else getCloseUserTextField().setText("");

		paymentsLabel.setText(((Long) fields.get("paymentsNumber")).toString());
		accountsLabel.setText(((Long) fields.get("accountsNumber")).toString());
		cardsLabel.setText(((Long) fields.get("cardsNumber")).toString());
		vouchersLabel.setText(((Long) fields.get("vouchersNumber")).toString());
		accountsAmountLabel.setText(((BigDecimal) fields.get("accountsTotal")).toString());
		cardsAmountLabel.setText(((BigDecimal) fields.get("cardsTotal")).toString());
		vouchersAmountLabel.setText(((BigDecimal) fields.get("vouchersTotal")).toString());
		totalLabel.setText(((BigDecimal) fields.get("cashTotal")).toString());

		paymentsNumber = ((Long) fields.get("paymentsNumber")).longValue();
		accountsNumber = ((Long) fields.get("accountsNumber")).longValue();
		cardsNumber = ((Long) fields.get("cardsNumber")).longValue();
		vouchersNumber = ((Long) fields.get("vouchersNumber")).longValue();
		accountsTotal = ((BigDecimal) fields.get("accountsTotal")).doubleValue();
		cardsTotal = ((BigDecimal) fields.get("cardsTotal")).doubleValue();
		vouchersTotal = ((BigDecimal) fields.get("vouchersTotal")).doubleValue();
		cashTotal = ((BigDecimal) fields.get("cashTotal")).doubleValue();

	}	

	private boolean inputErrors() {

		if (!ErrorsHandler.getErrors().isEmpty())  {

			if (ErrorsHandler.getErrorAt(0).getItem().equals("unaceptablePayment")) {
				getInfoPanel().setErrorText("<html>" + Locale.get("InfoPanel.errors.title") + "<br>" 
						+ Locale.get("CashWindow.errors.unaceptablePayment")+ "</html>");

				Toolkit.getDefaultToolkit().beep();
				return true;

			}

			if (ErrorsHandler.getErrorAt(0).getItem().equals("closedPayment")) {
				getInfoPanel().setErrorText("<html>" + Locale.get("InfoPanel.errors.title") + "<br>" 
						+ Locale.get("CashWindow.errors.closedPayment")+ "</html>");

				Toolkit.getDefaultToolkit().beep();
				return true;

			}

			if (ErrorsHandler.getErrorAt(0).getItem().equals("closedCash")) {
				getInfoPanel().setErrorText("<html>" + Locale.get("InfoPanel.errors.title") + "<br>" 
						+ Locale.get("CashWindow.errors.closedCash")+ "</html>");

				Toolkit.getDefaultToolkit().beep();
				return true;

			}

			
			if (ErrorsHandler.getErrorAt(0).getItem().equals("delete")) {
				getInfoPanel().setErrorText("<html>" + Locale.get("InfoPanel.errors.title") + "<br>" 
						+ Locale.get("CashWindow.errors.delete")+ "</html>");

				Toolkit.getDefaultToolkit().beep();
				return true;

			}
			
		}

		return false;

	}

}  //  @jve:decl-index=0:visual-constraint="10,10"
