/************************************************************************************************/
/*                                                                                              */
/*      <cyberHotel: Gestion Hotelera GPL>                                                      */
/*      Copyright (C) <2008> <davide05@gmail.com>                                               */
/*                                                                                              */
/*      This program is free software: you can redistribute it and/or modify                    */
/*      it under the terms of the GNU General Public License as published by                    */
/*      the Free Software Foundation, either version 3 of the License, or                       */
/*      (at your option) any later version.                                                     */
/*                                                                                              */
/*      This program is distributed in the hope that it will be useful,                         */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of                          */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                                    */ 
/*      See the GNU General Public License for more details.                                    */
/*                                                                                              */
/*      You should have received a copy of the GNU General Public License                       */
/*      along with this program.  If not, see <http://www.gnu.org/licenses/>.                   */
/*                                                                                              */
/************************************************************************************************/

package gz.cyberbrain.cyberhotel.view.swing.reception;

import gz.cyberbrain.cyberhotel.controller.AgencyListAdapter;
import gz.cyberbrain.cyberhotel.controller.BoardListAdapter;
import gz.cyberbrain.cyberhotel.controller.CompanyListAdapter;
import gz.cyberbrain.cyberhotel.controller.IndividualReservationListAdapter;
import gz.cyberbrain.cyberhotel.controller.ParticularListAdapter;
import gz.cyberbrain.cyberhotel.controller.PriceTypeListAdapter;
import gz.cyberbrain.cyberhotel.controller.RoomListAdapter;
import gz.cyberbrain.cyberhotel.controller.SaveAdapter;
import gz.cyberbrain.cyberhotel.controller.util.ErrorsHandler;
import gz.cyberbrain.cyberhotel.model.util.dates.DateOperations;
import gz.cyberbrain.cyberhotel.view.swing.ToolsWindow;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.DateFormat;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.InfoPanel;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.IntegerDocument;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.Payable;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.TextItem;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.ToolsPanel;
import gz.cyberbrain.cyberhotel.view.swing.util.locale.Locale;

import java.awt.Color;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.io.Serializable;
import java.text.ParseException;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Vector;

import javax.swing.JPanel;
import javax.swing.JInternalFrame;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.JComboBox;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JButton;
import javax.swing.UIManager;
import javax.swing.border.LineBorder;
import javax.swing.text.Document;

public class IndividualReservationWindow extends ToolsWindow implements Payable {

	private static final long serialVersionUID = 1L;
	private static IndividualReservationWindow individualReservationWindow = null;
	private Serializable id = null;  //  @jve:decl-index=0:
	private Serializable payerId = null;
	private SaveAdapter saveAdapter = null;  //  @jve:decl-index=0:
	private JInternalFrame owner = null;
	private String dialogMessage = null;
	private String dialogTitle = null;
	private JPanel jContentPane = null;
	private InfoPanel infoPanel = null;
	private JLabel reservatorNameLabel = null;
	private JTextField reservatorNameTextField = null;
	private JLabel reservatorCifLabel = null;
	private JTextField reservatorCifTextField = null;
	private JLabel reservatorPhoneLabel = null;
	private JTextField reservatorPhoneTextField = null;
	private JLabel incomingDateLabel = null;
	private JFormattedTextField incomingDateTextField = null;
	private JLabel expectedOutgoingDateLabel = null;
	private JFormattedTextField expectedOutgoingDateTextField = null;
	private JLabel trueOutgoingDateLabel = null;
	private JFormattedTextField trueOutgoingDateTextField = null;
	private JLabel payerLabel = null;
	private JLabel agencyLabel = null;
	private JComboBox agencyComboBox = null;
	private JLabel companyLabel = null;
	private JComboBox companyComboBox = null;
	private JLabel particularLabel = null;
	private JComboBox particularComboBox = null;
	private JLabel roomLabel = null;
	private JComboBox roomComboBox = null;
	private JLabel boardLabel = null;
	private JComboBox boardComboBox = null;
	private JLabel priceTypeLabel = null;
	private JComboBox priceTypeComboBox = null;
	private JLabel notesLabel = null;
	private JScrollPane notesScrollPane = null;
	private JTextArea notesTextArea = null;
	private JTextField payerTextField = null;
	private JButton payerButton = null;
	private JButton saveButton = null;
	private JButton cancelButton = null;
	private JLabel reservatorAccountLabel = null;
	private JTextField reservatorAccountTextField = null;
	private JLabel reservatorCardLabel = null;
	private JTextField reservatorCardTextField = null;
	private JLabel statusLabel = null;
	private JComboBox statusComboBox = null;
	private ToolsPanel toolsPanel = null;
	
	/**
	 * This is the xxx default constructor
	 */
	private IndividualReservationWindow() {
		super("", false, true, false, true);
		initialize();
	}

	public static IndividualReservationWindow getInstance() {
		if (individualReservationWindow == null) individualReservationWindow = new IndividualReservationWindow();
		return individualReservationWindow;
	}

	public void setAdapter(SaveAdapter saveAdapter) {
		this.saveAdapter = saveAdapter;
	}

	public SaveAdapter getAdapter() {
		return saveAdapter;
	}

	public JInternalFrame getOwner() {
		return owner;
	}

	public void setOwner(JInternalFrame owner) {
		this.owner = owner;
	}

	public String getDialogMessage() {
		return dialogMessage;
	}

	public void setDialogMessage(String dialogMessage) {
		this.dialogMessage = dialogMessage;
	}

	public String getDialogTitle() {
		return dialogTitle;
	}

	public void setDialogTitle(String dialogTitle) {
		this.dialogTitle = dialogTitle;
	}

	public void setId(Serializable id) {
		this.id = id;
	}

	public Serializable getId() {
		return id;
	}

	public void setPayerId(Serializable payerId) {
		this.payerId = payerId;
	}

	public Serializable getPayerId() {
		return payerId;
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(710, 508);
		this.setContentPane(getJContentPane());
		this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		this.addInternalFrameListener(new javax.swing.event.InternalFrameAdapter()
		{

			public void internalFrameClosed(javax.swing.event.InternalFrameEvent evt)
			{
				(IndividualReservationWindow.getInstance()).clean();
				(IndividualReservationWindow.getInstance()).lock();
				getToolsPanel().reset();
				individualReservationWindow = null;
			}
			
		});
		lock();
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			statusLabel = new JLabel();
			statusLabel.setBounds(new Rectangle(395, 66, 49, 16));
			statusLabel.setText(Locale.get("IndividualReservationWindow.statusLabel"));
			reservatorCardLabel = new JLabel();
			reservatorCardLabel.setBounds(new Rectangle(11, 141, 71, 16));
			reservatorCardLabel.setText(Locale.get("IndividualReservationWindow.reservatorCardLabel"));
			reservatorAccountLabel = new JLabel();
			reservatorAccountLabel.setBounds(new Rectangle(11, 116, 71, 16));
			reservatorAccountLabel.setText(Locale.get("IndividualReservationWindow.reservatorAccountLabel"));
			notesLabel = new JLabel();
			notesLabel.setBounds(new Rectangle(420, 223, 38, 16));
			notesLabel.setText(Locale.get("IndividualReservationWindow.notesLabel"));
			priceTypeLabel = new JLabel();
			priceTypeLabel.setBounds(new Rectangle(11, 388, 54, 16));
			priceTypeLabel.setText(Locale.get("IndividualReservationWindow.priceTypeLabel"));
			boardLabel = new JLabel();
			boardLabel.setBounds(new Rectangle(11, 361, 54, 16));
			boardLabel.setText(Locale.get("IndividualReservationWindow.boardLabel"));
			roomLabel = new JLabel();
			roomLabel.setBounds(new Rectangle(11, 333, 54, 16));
			roomLabel.setText(Locale.get("IndividualReservationWindow.roomLabel"));
			particularLabel = new JLabel();
			particularLabel.setBounds(new Rectangle(11, 301, 54, 16));
			particularLabel.setText(Locale.get("IndividualReservationWindow.particularLabel"));
			companyLabel = new JLabel();
			companyLabel.setBounds(new Rectangle(11, 243, 54, 16));
			companyLabel.setText(Locale.get("IndividualReservationWindow.companyLabel"));
			agencyLabel = new JLabel();
			agencyLabel.setBounds(new Rectangle(11, 214, 54, 16));
			agencyLabel.setText(Locale.get("IndividualReservationWindow.agencyLabel"));
			payerLabel = new JLabel();
			payerLabel.setBounds(new Rectangle(11, 182, 54, 16));
			payerLabel.setText(Locale.get("IndividualReservationWindow.payerLabel"));
			trueOutgoingDateLabel = new JLabel();
			trueOutgoingDateLabel.setBounds(new Rectangle(537, 66, 74, 16));
			trueOutgoingDateLabel.setText(Locale.get("IndividualReservationWindow.trueOutgoingDateLabel"));
			expectedOutgoingDateLabel = new JLabel();
			expectedOutgoingDateLabel.setBounds(new Rectangle(537, 93, 74, 16));
			expectedOutgoingDateLabel.setText(Locale.get("IndividualReservationWindow.expectedOutgoingDateLabel"));
			incomingDateLabel = new JLabel();
			incomingDateLabel.setBounds(new Rectangle(395, 93, 49, 16));
			incomingDateLabel.setText(Locale.get("IndividualReservationWindow.incomingDateLabel"));
			reservatorPhoneLabel = new JLabel();
			reservatorPhoneLabel.setBounds(new Rectangle(192, 92, 35, 16));
			reservatorPhoneLabel.setText(Locale.get("IndividualReservationWindow.reservatorPhoneLabel"));
			reservatorCifLabel = new JLabel();
			reservatorCifLabel.setBounds(new Rectangle(11, 92, 35, 16));
			reservatorCifLabel.setText(Locale.get("IndividualReservationWindow.reservatorCifLabel"));
			reservatorNameLabel = new JLabel();
			reservatorNameLabel.setBounds(new Rectangle(11, 66, 71, 16));
			reservatorNameLabel.setText(Locale.get("IndividualReservationWindow.reservatorNameLabel"));
			jContentPane = new JPanel();
			jContentPane.setLayout(null);
			jContentPane.add(getInfoPanel(), null);
			jContentPane.add(reservatorNameLabel, null);
			jContentPane.add(getReservatorNameTextField(), null);
			jContentPane.add(reservatorCifLabel, null);
			jContentPane.add(getReservatorCifTextField(), null);
			jContentPane.add(reservatorPhoneLabel, null);
			jContentPane.add(getReservatorPhoneTextField(), null);
			jContentPane.add(incomingDateLabel, null);
			jContentPane.add(getIncomingDateTextField(), null);
			jContentPane.add(expectedOutgoingDateLabel, null);
			jContentPane.add(getExpectedOutgoingDateTextField(), null);
			jContentPane.add(trueOutgoingDateLabel, null);
			jContentPane.add(getTrueOutgoingDateTextField(), null);
			jContentPane.add(payerLabel, null);
			jContentPane.add(agencyLabel, null);
			jContentPane.add(getAgencyComboBox(), null);
			jContentPane.add(companyLabel, null);
			jContentPane.add(getCompanyComboBox(), null);
			jContentPane.add(particularLabel, null);
			jContentPane.add(getParticularComboBox(), null);
			jContentPane.add(roomLabel, null);
			jContentPane.add(getRoomComboBox(), null);
			jContentPane.add(boardLabel, null);
			jContentPane.add(getBoardComboBox(), null);
			jContentPane.add(priceTypeLabel, null);
			jContentPane.add(getPriceTypeComboBox(), null);
			jContentPane.add(notesLabel, null);
			jContentPane.add(getNotesScrollPane(), null);
			jContentPane.add(getPayerTextField(), null);
			jContentPane.add(getPayerButton(), null);
			jContentPane.add(getSaveButton(), null);
			jContentPane.add(getCancelButton(), null);
			jContentPane.add(reservatorAccountLabel, null);
			jContentPane.add(getReservatorAccountTextField(), null);
			jContentPane.add(reservatorCardLabel, null);
			jContentPane.add(getReservatorCardTextField(), null);
			jContentPane.add(statusLabel, null);
			jContentPane.add(getStatusComboBox(), null);
			jContentPane.add(getToolsPanel(), null);
		}
		return jContentPane;
	}

	/**
	 * This method initializes infoPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private InfoPanel getInfoPanel() {
		if (infoPanel == null) {
			infoPanel = new InfoPanel();
			infoPanel.setBounds(new Rectangle(0, 0, this.getWidth(), 50));
		}
		return infoPanel;
	}

	public void setInfoPanel(InfoPanel ipanel) {		  

		jContentPane.remove(infoPanel);
		infoPanel = ipanel;
		jContentPane.add(infoPanel);

	}

	/**
	 * This method initializes reservatorNameTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getReservatorNameTextField() {
		if (reservatorNameTextField == null) {
			reservatorNameTextField = new JTextField();
			reservatorNameTextField.setBounds(new Rectangle(90, 65, 286, 20));
			reservatorNameTextField.setBackground(Color.PINK);
		}
		return reservatorNameTextField;
	}

	/**
	 * This method initializes reservatorCifTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getReservatorCifTextField() {
		if (reservatorCifTextField == null) {
			reservatorCifTextField = new JTextField();
			reservatorCifTextField.setBounds(new Rectangle(90, 92, 98, 20));
		}
		return reservatorCifTextField;
	}

	/**
	 * This method initializes reservatorPhoneTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getReservatorPhoneTextField() {
		if (reservatorPhoneTextField == null) {
			Document integerDocument = new IntegerDocument(15);
			reservatorPhoneTextField = new JTextField(integerDocument,"",15);
			reservatorPhoneTextField.setHorizontalAlignment(JTextField.TRAILING);			
			reservatorPhoneTextField.setBounds(new Rectangle(235, 92, 141, 20));
		}
		return reservatorPhoneTextField;
	}

	/**
	 * This method initializes incomingDateTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JFormattedTextField getIncomingDateTextField() {
		if (incomingDateTextField == null) {
			try {
				incomingDateTextField = new JFormattedTextField(new DateFormat());
			} catch (ParseException e) {				
				e.printStackTrace();
			}
			incomingDateTextField.setBounds(new Rectangle(451, 93, 85, 20));
		}
		return incomingDateTextField;
	}

	/**
	 * This method initializes expectedOutgoingDateTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JFormattedTextField getExpectedOutgoingDateTextField() {
		if (expectedOutgoingDateTextField == null) {
			try {
				Calendar date = Calendar.getInstance();
				date.add(Calendar.DATE, 1);
				expectedOutgoingDateTextField = new JFormattedTextField(new DateFormat());
				expectedOutgoingDateTextField.setText(DateOperations.calendarToText(date));
				expectedOutgoingDateTextField.addFocusListener(new java.awt.event.FocusAdapter() {
					public void focusLost(java.awt.event.FocusEvent e) {
						getTrueOutgoingDateTextField().setText(DateOperations.calendarToText(DateOperations.textToCalendar(expectedOutgoingDateTextField.getText())));					
					}
				});

			} catch (ParseException e) {
				e.printStackTrace();
			}
			expectedOutgoingDateTextField.setBounds(new Rectangle(616, 93, 75, 20));
		}
		return expectedOutgoingDateTextField;
	}

	/**
	 * This method initializes trueOutgoingDateTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JFormattedTextField getTrueOutgoingDateTextField() {
		if (trueOutgoingDateTextField == null) {
			try {
				Calendar date = Calendar.getInstance();
				date.add(Calendar.DATE, 1);
				trueOutgoingDateTextField = new JFormattedTextField(new DateFormat());
				trueOutgoingDateTextField.setText(DateOperations.calendarToText(date));				
			} catch (ParseException e) {			
				e.printStackTrace();
			}
			trueOutgoingDateTextField.setBounds(new Rectangle(616, 66, 75, 20));
			trueOutgoingDateTextField.setEditable(false);
		}
		return trueOutgoingDateTextField;
	}

	/**
	 * This method initializes agencyComboBox	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	@SuppressWarnings("unchecked")
	private JComboBox getAgencyComboBox() {
		if (agencyComboBox == null) {
			Vector agencies = (new AgencyListAdapter()).load();
			agencies.insertElementAt(new TextItem(new Long(0),""), 0);
			agencyComboBox = new JComboBox(agencies);
			agencyComboBox.setBounds(new Rectangle(90, 212, 286, 22));
		}
		return agencyComboBox;
	}

	/**
	 * This method initializes companyComboBox	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	@SuppressWarnings("unchecked")
	private JComboBox getCompanyComboBox() {
		if (companyComboBox == null) {			
			Vector companies = (new CompanyListAdapter()).load();
			companies.insertElementAt(new TextItem(new Long(0),""), 0);
			companyComboBox = new JComboBox(companies);
			companyComboBox.setBounds(new Rectangle(90, 243, 286, 22));
		}
		return companyComboBox;
	}

	/**
	 * This method initializes particularComboBox	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	@SuppressWarnings("unchecked")
	private JComboBox getParticularComboBox() {
		if (particularComboBox == null) {
			Vector particulars = (new ParticularListAdapter()).load();
			particulars.insertElementAt(new TextItem(new Long(0),""), 0);
			particularComboBox = new JComboBox(particulars);
			particularComboBox.setBounds(new Rectangle(90, 300, 286, 22));
		}
		return particularComboBox;
	}

	/**
	 * This method initializes roomComboBox	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	@SuppressWarnings("unchecked")
	private JComboBox getRoomComboBox() {
		if (roomComboBox == null) {
			Vector rooms = (new RoomListAdapter()).load();
			rooms.insertElementAt(new TextItem(new String(""),""), 0);
			roomComboBox = new JComboBox(rooms);
			roomComboBox.setBounds(new Rectangle(90, 331, 143, 22));
		}
		return roomComboBox;
	}

	/**
	 * This method initializes boardComboBox	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	@SuppressWarnings("unchecked")
	private JComboBox getBoardComboBox() {
		if (boardComboBox == null) {
			Vector boards = (new BoardListAdapter()).load();
			boards.insertElementAt(new TextItem(new String(""),""), 0);
			boardComboBox = new JComboBox(boards);
			boardComboBox.setBounds(new Rectangle(90, 359, 143, 22));
		}
		return boardComboBox;
	}

	/**
	 * This method initializes priceTypeComboBox	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	@SuppressWarnings("unchecked")
	private JComboBox getPriceTypeComboBox() {
		if (priceTypeComboBox == null) {
			Vector priceTypes = (new PriceTypeListAdapter()).load();
			priceTypes.insertElementAt(new TextItem(new String(""),""), 0);
			priceTypeComboBox = new JComboBox(priceTypes);
			priceTypeComboBox.setBounds(new Rectangle(90, 386, 143, 22));
		}
		return priceTypeComboBox;
	}

	/**
	 * This method initializes notesScrollPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getNotesScrollPane() {
		if (notesScrollPane == null) {
			notesScrollPane = new JScrollPane();
			notesScrollPane.setBounds(new Rectangle(420, 244, 269, 138));
			notesScrollPane.setViewportView(getNotesTextArea());
		}
		return notesScrollPane;
	}

	/**
	 * This method initializes notesTextArea	
	 * 	
	 * @return javax.swing.JTextArea	
	 */
	private JTextArea getNotesTextArea() {
		if (notesTextArea == null) {
			notesTextArea = new JTextArea();
		}
		return notesTextArea;
	}

	/**
	 * This method initializes payerTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getPayerTextField() {
		if (payerTextField == null) {
			payerTextField = new JTextField();
			payerTextField.setBounds(new Rectangle(90, 182, 286, 22));
			payerTextField.setEditable(false);
		}
		return payerTextField;
	}

	/**
	 * This method initializes payerButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getPayerButton() {
		if (payerButton == null) {
			payerButton = new JButton();
			payerButton.setText(Locale.get("IndividualReservationWindow.payerButton"));
			payerButton.setBounds(new Rectangle(385, 182, 27, 20));
			payerButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {

					PayerWindow payerWindow = new PayerWindow();
					payerWindow.setObserver(IndividualReservationWindow.getInstance());
					payerWindow.setVisible(true);

				}
			});

		}
		return payerButton;
	}

	public void setPayer(Serializable payerId, String payerText) {
		getPayerTextField().setText(payerText);
		setPayerId(payerId);
	}

	/**
	 * This method initializes saveButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getSaveButton() {
		if (saveButton == null) {
			saveButton = new JButton();
			saveButton.setText(Locale.get("IndividualReservationWindow.saveButton"));
			saveButton.setBounds(new Rectangle(523, 397, 76, 17));
			saveButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {

					HashMap<String, Serializable> attrs = new HashMap<String, Serializable>();					

					attrs.put("id", (Long) getId());
					attrs.put("reservatorName", getReservatorNameTextField().getText());					           
					attrs.put("reservatorCif", getReservatorCifTextField().getText()); 
					attrs.put("reservatorPhone", getReservatorPhoneTextField().getText());
					attrs.put("reservatorAccount", getReservatorAccountTextField().getText());
					attrs.put("reservatorCard", getReservatorCardTextField().getText());

					String status = null;

					switch (getStatusComboBox().getSelectedIndex()) {
					case 0:
						status = "Pendente";
						break;
					case 1:
						status = "Rexistrada";
						break;
					case 2:
						status = "Anulada";
						break;
					case 3:
						status = "Finalizada";
						break;
					}

					attrs.put("reservationStatus", status); 
					attrs.put("incomingDate", DateOperations.textToCalendar(getIncomingDateTextField().getText()));

					attrs.put("payer", (IndividualReservationWindow.getInstance()).getPayerId());					

					if (getAgencyComboBox().getItemCount() != 0 && ((Long) ((TextItem) getAgencyComboBox().getSelectedItem()).getId() != 0)) attrs.put("agency", ((TextItem) getAgencyComboBox().getSelectedItem()).getId());
					else attrs.put("agency", null);

					if (getCompanyComboBox().getItemCount() != 0 && ((Long) ((TextItem) getCompanyComboBox().getSelectedItem()).getId() != 0)) attrs.put("company", ((TextItem) getCompanyComboBox().getSelectedItem()).getId());
					else attrs.put("company", null);

					attrs.put("notes", getNotesTextArea().getText()); 
					attrs.put("expectedOutgoingDate", DateOperations.textToCalendar(getExpectedOutgoingDateTextField().getText()));
					attrs.put("trueOutgoingDate", DateOperations.textToCalendar(getTrueOutgoingDateTextField().getText()));
					attrs.put("byHours", new Boolean(false));

					Vector <HashMap> requests = new Vector <HashMap> ();
					
					HashMap <String, Serializable> request = new HashMap <String, Serializable> ();
					
					if (getParticularComboBox().getItemCount() != 0) 						
						request.put("particular", (Long) ((TextItem) getParticularComboBox().getSelectedItem()).getId());
					else request.put("particular", null);						

					if (getRoomComboBox().getItemCount() != 0) request.put("room", (String) ((TextItem) getRoomComboBox().getSelectedItem()).getId());
					else request.put("room", new Long(0));

					if (getBoardComboBox().getItemCount() != 0) request.put("board", (String) ((TextItem) getBoardComboBox().getSelectedItem()).getId());
					else request.put("board", new Long(0));

					if (getPriceTypeComboBox().getItemCount() != 0) request.put("priceType", (String) ((TextItem) getPriceTypeComboBox().getSelectedItem()).getId());
					else request.put("priceType", new Long(0));

					requests.add(request);										

					setId(getAdapter().save(attrs, requests));

					if (!inputErrors()) {

						setDialogMessage(Locale.get("JOptionPane.IndividualReservationWindow.saveButton.message"));
						setDialogTitle(Locale.get("JOptionPane.IndividualReservationWindow.saveButton.title"));

						Object[] options = {Locale.get("JOptionPane.Ok")};
						int n = JOptionPane.showOptionDialog(IndividualReservationWindow.getInstance(),
								getDialogMessage(),
								getDialogTitle(),
								JOptionPane.OK_OPTION,
								JOptionPane.INFORMATION_MESSAGE,
								null,
								options,
								options[0]);

						if (n == JOptionPane.OK_OPTION) {							
							clean();
							lock();
							getToolsPanel().current();							
						}																	

					}

				}

			});

		}
		return saveButton;
	}

	/**
	 * This method initializes cancelButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton();
			cancelButton.setText(Locale.get("IndividualReservationWindow.cancelButton"));
			cancelButton.setBounds(new Rectangle(610, 397, 77, 17));
			cancelButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {

					(IndividualReservationWindow.getInstance()).clean();
					(IndividualReservationWindow.getInstance()).lock();
					getToolsPanel().reset();
					(IndividualReservationWindow.getInstance()).dispose();
					individualReservationWindow = null;

				}
			});

		}
		return cancelButton;
	}

	/**
	 * This method initializes reservatorAccountTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getReservatorAccountTextField() {
		if (reservatorAccountTextField == null) {
			reservatorAccountTextField = new JTextField();
			reservatorAccountTextField.setHorizontalAlignment(JTextField.TRAILING);
			reservatorAccountTextField.setBounds(new Rectangle(90, 116, 286, 20));
		}
		return reservatorAccountTextField;
	}

	/**
	 * This method initializes reservatorCardTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getReservatorCardTextField() {
		if (reservatorCardTextField == null) {
			Document integerDocument = new IntegerDocument(16);
			reservatorCardTextField = new JTextField(integerDocument, "", 16);
			reservatorCardTextField.setHorizontalAlignment(JTextField.TRAILING);	
			reservatorCardTextField.setBounds(new Rectangle(90, 141, 286, 20));
		}
		return reservatorCardTextField;
	}

	/**
	 * This method initializes statusComboBox	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	private JComboBox getStatusComboBox() {
		if (statusComboBox == null) {
			statusComboBox = new JComboBox();			
			statusComboBox.setBounds(new Rectangle(450, 66, 80, 20));
			statusComboBox.addItem(Locale.get("IndividualReservationWindow.statusCombo.item0"));
			statusComboBox.addItem(Locale.get("IndividualReservationWindow.statusCombo.item1"));
			statusComboBox.addItem(Locale.get("IndividualReservationWindow.statusCombo.item2"));
			statusComboBox.addItem(Locale.get("IndividualReservationWindow.statusCombo.item3"));
		}
		return statusComboBox;
	}

	public void clean() {		

		getReservatorNameTextField().setText("");
		getReservatorCifTextField().setText("");
		getReservatorPhoneTextField().setText("");
		getReservatorAccountTextField().setText("");
		getReservatorCardTextField().setText("");
		getIncomingDateTextField().setText(DateOperations.calendarToText(Calendar.getInstance()));
		if (getStatusComboBox().getItemCount() != 0) getStatusComboBox().setSelectedIndex(0);
		Calendar date = Calendar.getInstance();
		date.add(Calendar.DATE, 1);
		getExpectedOutgoingDateTextField().setText(DateOperations.calendarToText(date));
		getTrueOutgoingDateTextField().setText(DateOperations.calendarToText(date));
		getPayerTextField().setText("");
		setPayer(null,null);
		if (getAgencyComboBox().getItemCount() != 0) getAgencyComboBox().setSelectedIndex(0);
		if (getCompanyComboBox().getItemCount() != 0) getCompanyComboBox().setSelectedIndex(0);
		if (getParticularComboBox().getItemCount() != 0) getParticularComboBox().setSelectedIndex(0);
		if (getRoomComboBox().getItemCount() != 0) getRoomComboBox().setSelectedIndex(0);
		if (getBoardComboBox().getItemCount() != 0) getBoardComboBox().setSelectedIndex(0);
		if (getPriceTypeComboBox().getItemCount() != 0) getPriceTypeComboBox().setSelectedIndex(0);
		getNotesTextArea().setText("");

		cleanErrors();
	}

	public void fill(HashMap fields) {

		setId((Long) fields.get("id"));
		getReservatorNameTextField().setText((String) fields.get("reservatorName"));
		getReservatorCifTextField().setText((String) fields.get("reservatorCif"));
		getReservatorPhoneTextField().setText((String) fields.get("reservatorPhone"));
		getReservatorAccountTextField().setText((String) fields.get("reservatorAccount"));
		getReservatorCardTextField().setText((String) fields.get("reservatorCard"));
		getIncomingDateTextField().setText(DateOperations.calendarToText((Calendar) fields.get("incomingDate")));
		getExpectedOutgoingDateTextField().setText(DateOperations.calendarToText((Calendar) fields.get("expectedOutgoingDate")));
		getTrueOutgoingDateTextField().setText(DateOperations.calendarToText((Calendar) fields.get("trueOutgoingDate")));

		getPayerTextField().setText(((TextItem) fields.get("payer")).getText());
		setPayer(((TextItem) fields.get("payer")).getId(),((TextItem) fields.get("payer")).getText());

		if (((String) fields.get("reservationStatus")).equals("Pendente")) getStatusComboBox().setSelectedIndex(0);
		if (((String) fields.get("reservationStatus")).equals("Rexistrada")) getStatusComboBox().setSelectedIndex(1);
		if (((String) fields.get("reservationStatus")).equals("Anulada")) getStatusComboBox().setSelectedIndex(2);
		if (((String) fields.get("reservationStatus")).equals("Finalizada")) getStatusComboBox().setSelectedIndex(3);

		for (int i=0; i<getAgencyComboBox().getItemCount(); i++) {
			TextItem item = (TextItem) getAgencyComboBox().getItemAt(i); 
			if ((item.getId()).equals(((TextItem) fields.get("agency")).getId())) {
				getAgencyComboBox().setSelectedIndex(i);
			}
		}

		for (int i=0; i<getCompanyComboBox().getItemCount(); i++) {
			TextItem item = (TextItem) getCompanyComboBox().getItemAt(i); 
			if ((item.getId()).equals(((TextItem) fields.get("company")).getId())) {
				getCompanyComboBox().setSelectedIndex(i);
			}
		}
		
		for (int i=0; i<getParticularComboBox().getItemCount(); i++) {
			TextItem item = (TextItem) getParticularComboBox().getItemAt(i); 
			if ((item.getId()).equals(((TextItem) fields.get("particular")).getId())) {
				getParticularComboBox().setSelectedIndex(i);
			}
		}

		for (int i=0; i<getRoomComboBox().getItemCount(); i++) {
			TextItem item = (TextItem) getRoomComboBox().getItemAt(i); 
			if ((item.getId()).equals(((TextItem) fields.get("room")).getId())) {
				getRoomComboBox().setSelectedIndex(i);
			}
		}

		for (int i=0; i<getBoardComboBox().getItemCount(); i++) {
			TextItem item = (TextItem) getBoardComboBox().getItemAt(i); 
			if ((item.getId()).equals(((TextItem) fields.get("board")).getId())) {
				getBoardComboBox().setSelectedIndex(i);
			}
		}

		for (int i=0; i<getPriceTypeComboBox().getItemCount(); i++) {
			TextItem item = (TextItem) getPriceTypeComboBox().getItemAt(i); 
			if ((item.getId()).equals(((TextItem) fields.get("priceType")).getId())) {
				getPriceTypeComboBox().setSelectedIndex(i);
			}
		}

		getNotesTextArea().setText((String) fields.get("notes"));

	}
	
	/**
	 * This method initializes toolsPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private ToolsPanel getToolsPanel() {
		if (toolsPanel == null) {
			toolsPanel = new ToolsPanel(this, new IndividualReservationListAdapter());
			toolsPanel.setBounds(new Rectangle(0, 420, this.getWidth(), 50));
		}
		return toolsPanel;
	}

	public void lock() {

		getReservatorNameTextField().setEditable(false);
		getReservatorCifTextField().setEditable(false);
		getReservatorPhoneTextField().setEditable(false);
		getReservatorAccountTextField().setEditable(false);
		getReservatorCardTextField().setEditable(false);
		getIncomingDateTextField().setEditable(false);
		getExpectedOutgoingDateTextField().setEditable(false);

		getPayerTextField().setEditable(false);
		getPayerButton().setEnabled(false);		

		getStatusComboBox().setEnabled(false);		
		getAgencyComboBox().setEnabled(false);		
		getCompanyComboBox().setEnabled(false);
		getParticularComboBox().setEnabled(false);		
		getRoomComboBox().setEnabled(false);		
		getBoardComboBox().setEnabled(false);		
		getPriceTypeComboBox().setEnabled(false);
		getNotesTextArea().setEditable(false);

		getSaveButton().setEnabled(false);

	}

	public void unlock() {

		getReservatorNameTextField().setEditable(true);
		getReservatorCifTextField().setEditable(true);
		getReservatorPhoneTextField().setEditable(true);
		getReservatorAccountTextField().setEditable(true);
		getReservatorCardTextField().setEditable(true);
		getIncomingDateTextField().setEditable(true);
		getExpectedOutgoingDateTextField().setEditable(true);

		getPayerButton().setEnabled(true);		

		getStatusComboBox().setEnabled(true);		
		getAgencyComboBox().setEnabled(true);		
		getCompanyComboBox().setEnabled(true);
		getParticularComboBox().setEnabled(true);		
		getRoomComboBox().setEnabled(true);		
		getBoardComboBox().setEnabled(true);		
		getPriceTypeComboBox().setEnabled(true);
		getNotesTextArea().setEditable(true);

		getSaveButton().setEnabled(true);

	}

	public void cleanErrors() {
		
		getInfoPanel().setInfoText(getInfoPanel().getInfoText());
		
		getReservatorNameTextField().setBorder(UIManager.getBorder("TextField.border"));
		getIncomingDateTextField().setBorder(UIManager.getBorder("TextField.border"));
		getExpectedOutgoingDateTextField().setBorder(UIManager.getBorder("TextField.border"));
		getPayerTextField().setBorder(UIManager.getBorder("TextField.border"));
		getCompanyComboBox().setBorder(UIManager.getBorder("ComboBox.border"));
		getParticularComboBox().setBorder(UIManager.getBorder("ComboBox.border"));
		getRoomComboBox().setBorder(UIManager.getBorder("ComboBox.border"));
		getBoardComboBox().setBorder(UIManager.getBorder("ComboBox.border"));
		getPriceTypeComboBox().setBorder(UIManager.getBorder("ComboBox.border"));
		getAgencyComboBox().setBorder(UIManager.getBorder("ComboBox.border"));
		getStatusComboBox().setBorder(UIManager.getBorder("ComboBox.border"));		
	}	
	
	private boolean inputErrors() {		
		
		if (!ErrorsHandler.getErrors().isEmpty()) {

			if (ErrorsHandler.getErrorAt(0).getItem().equals("reservatorName")) {
				getInfoPanel().setErrorText("<html>" + Locale.get("InfoPanel.errors.title") + "<br>" 
						+ Locale.get("IndividualReservationWindow.errors.reservatorName")+ "</html>");

				getReservatorNameTextField().setBorder(new LineBorder(Color.red, 2));
				Toolkit.getDefaultToolkit().beep();
				return true;

			}
						
			if (ErrorsHandler.getErrorAt(0).getItem().equals("interval")) {
				getInfoPanel().setErrorText("<html>" + Locale.get("InfoPanel.errors.title") + "<br>" 
						+ Locale.get("IndividualReservationWindow.errors.interval")+ "</html>");

				getIncomingDateTextField().setBorder(new LineBorder(Color.red, 2));
				getExpectedOutgoingDateTextField().setBorder(new LineBorder(Color.red, 2));
				Toolkit.getDefaultToolkit().beep();
				return true;

			}

			if (ErrorsHandler.getErrorAt(0).getItem().equals("company")) {
				getInfoPanel().setErrorText("<html>" + Locale.get("InfoPanel.errors.title") + "<br>" 
						+ Locale.get("IndividualReservationWindow.errors.company")+ "</html>");
				getCompanyComboBox().setBorder(new LineBorder(Color.red, 2));			
				Toolkit.getDefaultToolkit().beep();
				return true;
			}

			if (ErrorsHandler.getErrorAt(0).getItem().equals("particular")) {
				getInfoPanel().setErrorText("<html>" + Locale.get("InfoPanel.errors.title") + "<br>" 
						+ Locale.get("IndividualReservationWindow.errors.particular")+ "</html>");

				getParticularComboBox().setBorder(new LineBorder(Color.red, 2));
				Toolkit.getDefaultToolkit().beep();
				return true;

			}

			if (ErrorsHandler.getErrorAt(0).getItem().equals("board")) {
				getInfoPanel().setErrorText("<html>" + Locale.get("InfoPanel.errors.title") + "<br>" 
						+ Locale.get("IndividualReservationWindow.errors.board")+ "</html>");

				getBoardComboBox().setBorder(new LineBorder(Color.red, 2));
				Toolkit.getDefaultToolkit().beep();
				return true;

			}

			if (ErrorsHandler.getErrorAt(0).getItem().equals("priceType")) {
				getInfoPanel().setErrorText("<html>" + Locale.get("InfoPanel.errors.title") + "<br>" 
						+ Locale.get("IndividualReservationWindow.errors.priceType")+ "</html>");

				getPriceTypeComboBox().setBorder(new LineBorder(Color.red, 2));
				Toolkit.getDefaultToolkit().beep();
				return true;

			}

			if (ErrorsHandler.getErrorAt(0).getItem().equals("payer")) {
				getInfoPanel().setErrorText("<html>" + Locale.get("InfoPanel.errors.title") + "<br>" 
						+ Locale.get("IndividualReservationWindow.errors.payer")+ "</html>");

				getPayerTextField().setBorder(new LineBorder(Color.red, 2));
				Toolkit.getDefaultToolkit().beep();
				return true;

			}

			if (ErrorsHandler.getErrorAt(0).getItem().equals("agency")) {
				getInfoPanel().setErrorText("<html>" + Locale.get("InfoPanel.errors.title") + "<br>" 
						+ Locale.get("IndividualReservationWindow.errors.agency")+ "</html>");

				getAgencyComboBox().setBorder(new LineBorder(Color.red, 2));
				Toolkit.getDefaultToolkit().beep();
				return true;

			}

			if (ErrorsHandler.getErrorAt(0).getItem().equals("roomUnaceptable")) {
				getInfoPanel().setErrorText("<html>" + Locale.get("InfoPanel.errors.title") + "<br>" 
						+ Locale.get("IndividualReservationWindow.errors.roomUnaceptable")+ "</html>");

				getRoomComboBox().setBorder(new LineBorder(Color.red, 2));
				Toolkit.getDefaultToolkit().beep();
				return true;

			}

			if (ErrorsHandler.getErrorAt(0).getItem().equals("roomUnavailable")) {
				getInfoPanel().setErrorText("<html>" + Locale.get("InfoPanel.errors.title") + "<br>" 
						+ Locale.get("IndividualReservationWindow.errors.roomUnavailable")+ "</html>");

				getRoomComboBox().setBorder(new LineBorder(Color.red, 2));
				Toolkit.getDefaultToolkit().beep();
				return true;

			}

			if (ErrorsHandler.getErrorAt(0).getItem().equals("delete")) {
				getInfoPanel().setErrorText("<html>" + Locale.get("InfoPanel.errors.title") + "<br>" 
						+ Locale.get("IndividualReservationWindow.errors.delete")+ "</html>");

				getStatusComboBox().setBorder(new LineBorder(Color.red, 2));
				Toolkit.getDefaultToolkit().beep();
				return true;

			} 
			
			if (ErrorsHandler.getErrorAt(0).getItem().equals("reservationStatus")) {
				getInfoPanel().setErrorText("<html>" + Locale.get("InfoPanel.errors.title") + "<br>" 
						+ Locale.get("IndividualReservationWindow.errors.status")+ "</html>");

				getStatusComboBox().setBorder(new LineBorder(Color.red, 2));
				Toolkit.getDefaultToolkit().beep();
				return true;

			} 
			
		}

		return false;

	}

	public boolean showErrors() {
		return inputErrors();
	}

	@Override
	public void update() {}
	
}  //  @jve:decl-index=0:visual-constraint="10,10"
