/************************************************************************************************/
/*                                                                                              */
/*      <cyberHotel: Gestion Hotelera GPL>                                                      */
/*      Copyright (C) <2008> <davide05@gmail.com>                                               */
/*                                                                                              */
/*      This program is free software: you can redistribute it and/or modify                    */
/*      it under the terms of the GNU General Public License as published by                    */
/*      the Free Software Foundation, either version 3 of the License, or                       */
/*      (at your option) any later version.                                                     */
/*                                                                                              */
/*      This program is distributed in the hope that it will be useful,                         */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of                          */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                                    */ 
/*      See the GNU General Public License for more details.                                    */
/*                                                                                              */
/*      You should have received a copy of the GNU General Public License                       */
/*      along with this program.  If not, see <http://www.gnu.org/licenses/>.                   */
/*                                                                                              */
/************************************************************************************************/

package gz.cyberbrain.cyberhotel.view.swing.util.gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Toolkit;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JPanel;
import javax.swing.RootPaneContainer;

public class GlassPane extends JPanel implements MouseListener {
  
	private static final long serialVersionUID = 1L;

	//RootPaneContainer interface
    private RootPaneContainer m_rootPane = null;
    
    //Stores the previous Glass Pane Component
    private Component m_prevGlassPane = null;
    private boolean m_handleMouseEvents = false;
    private boolean m_drawing = false;    

    /**
    * Constructor
    *
    * @param title
    * @exception
    */
    public GlassPane()
    {
    }
    
    //Mouse Events
    public void mousePressed(MouseEvent e) 
    {
        //sound beep
        Toolkit.getDefaultToolkit().beep(); 
    }
    
    public void mouseReleased(MouseEvent e) 
    {
    }
    
    public void mouseClicked(MouseEvent e) 
    {
    }
    
    public void mouseEntered(MouseEvent e) 
    {
    }
    
    public void mouseExited(MouseEvent e) 
    {
    }
    
    /**
    * Set the glassPane
    */
    public void setGlassPane(RootPaneContainer rootPane)
    {
        m_rootPane = rootPane;
        
        //store the current glass pane
        m_prevGlassPane = m_rootPane.getGlassPane();
        
        //set this as new glass pane
        m_rootPane.setGlassPane(this);
        
        //set opaque to false, i.e. make transparent
        setOpaque(false);
    }
    
    /**
    * remove the glassPane
    */
    public void removeGlassPane()
    {
        //set the glass pane visible false
        setVisible(false);
        
        //reset the previous glass pane
        m_rootPane.setGlassPane(m_prevGlassPane);
    }
    
    /**
    * Set the handleMoveEvents
    * This will add the mouseListener and all the event will be
    * trapped my the glass pane
    */
    public void setHandleMouseEvents(boolean handleMouseEvents)
    {
        if (m_handleMouseEvents == handleMouseEvents)
        {
            //ignore if the state is same
            return;
        }
        
        m_handleMouseEvents = handleMouseEvents;
        
        if (m_handleMouseEvents)
        {
            //add this as mouse event listener and trap all the
            //mouse events
            addMouseListener(this);
        }
        else
        {
            //remove mouse events listener
            removeMouseListener(this);
        }
        
        //This is important otherwise the glass pane will no catch events
        setVisible(true);
    }
        
    /**
    * setDrawing
    *
    */
    public void setDrawing(boolean drawing)
    {
        m_drawing = drawing;
        
        //This is important otherwise the glass pane will not be visible
        setVisible(true);
        
        //Call repaint
        repaint();
    }
    
    /**
    * get the drawing state
    */
    public boolean getDrawing()
    {
        return m_drawing;
    }
    
    /**
    * Handling Paint
    */
    public void paint(Graphics g)
    {
        super.paint(g);
        
        if (m_drawing)
        {
            g.setColor(Color.cyan);
            g.fillRoundRect(100, 100, 250, 200, 40, 40);
                        
            g.setFont(new Font("Arial", Font.BOLD, 12));
            g.setColor(Color.black);
            
            g.drawString("This example is from JavaReference.com", 110, 210);
        }
    }
} 
